@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 /*
@@ -70,7 +65,8 @@ static ParsedAttributeWrapper *parse_config_file(char *buf,
 
 static int process_config_file_attributes(const char *file,
                                           ParsedAttributeWrapper *w,
-                                          const char *display_name);
+                                          const char *display_name,
+                                          CtrlHandlesArray *handles_array);
 
 static void save_gui_parsed_attributes(ParsedAttributeWrapper *w,
                                        ParsedAttribute *p);
@@ -84,6 +80,11 @@ static int parse_config_property(const char *file, const char *line,
 static void write_config_properties(FILE *stream, ConfigProperties *conf,
                                     char *locale);
 
+static char *create_display_device_target_string(CtrlHandleTarget *t,
+                                                 ConfigProperties *conf);
+
+extern int __verbosity;
+extern int __verbosity_level_changed;
 
 /*
  * nv_read_config_file() - read the specified config file, building a
@@ -104,7 +105,8 @@ static void write_config_properties(FILE *stream, ConfigProperties *conf,
  */
 
 int nv_read_config_file(const char *file, const char *display_name,
-                        ParsedAttribute *p, ConfigProperties *conf)
+                        ParsedAttribute *p, ConfigProperties *conf,
+                        CtrlHandlesArray *handles_array)
 {
     int fd, ret, length;
     struct stat stat_buf;
@@ -183,7 +185,8 @@ int nv_read_config_file(const char *file, const char *display_name,
 
     /* process the parsed attributes */
 
-    ret = process_config_file_attributes(file, w, display_name);
+    ret = process_config_file_attributes(file, w, display_name, 
+                                         handles_array);
 
     /*
      * add any relevant parsed attributes back to the list to be
@@ -214,15 +217,14 @@ int nv_read_config_file(const char *file, const char *display_name,
 int nv_write_config_file(const char *filename, CtrlHandles *h,
                          ParsedAttribute *p, ConfigProperties *conf)
 {
-    int screen, ret, entry, bit, val;
+    int screen, ret, entry, val, display, randr_gamma_available;
     FILE *stream;
     time_t now;
-    AttributeTableEntry *a;
     ReturnStatus status;
+    NVCTRLAttributePermissionsRec perms;
     NVCTRLAttributeValidValuesRec valid;
-    uint32 mask;
     CtrlHandleTarget *t;
-    char *tmp_d_str, *prefix, scratch[4];
+    char *prefix, scratch[4];
     const char *tmp;
     char *locale = "C";
 
@@ -275,8 +277,8 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
     fprintf(stream, "\n");
 
     /*
-     * Note: we only save writable attributes addressable by X screen
-     * (i.e., we don't look at other target types, yet).
+     * Note: we only save writable attributes addressable by X screen here
+     * followed by attributes for display target types.
      */
     
     for (screen = 0; screen < h->targets[X_SCREEN_TARGET].n; screen++) {
@@ -289,7 +291,7 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
 
         /*
          * construct the prefix that will be printed in the config
-         * file infront of each attribute on this screen; this will
+         * file in front of each attribute on this screen; this will
          * either be "[screen]" or "[displayname]".
          */
 
@@ -304,9 +306,8 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
         /* loop over all the entries in the table */
 
         for (entry = 0; attributeTable[entry].name; entry++) {
+            const AttributeTableEntry *a = &attributeTable[entry];
 
-            a = &attributeTable[entry];
-            
             /* 
              * skip all attributes that are not supposed to be written
              * to the config file
@@ -321,6 +322,14 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
             
             if (a->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE) {
                 float c[3], b[3], g[3];
+
+                /*
+                 * if we are using RandR gamma, skip saving the color info
+                 */
+ 
+                status = NvCtrlGetAttribute(t->h, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+                if (status == NvCtrlSuccess && val) continue;
+
                 status = NvCtrlGetColorAttributes(t->h, c, b, g);
                 if (status != NvCtrlSuccess) continue;
                 fprintf(stream, "%s%c%s=%f\n",
@@ -328,60 +337,126 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
                         get_color_value(a->attr, c, b, g));
                 continue;
             }
-            
-            for (bit = 0; bit < 24; bit++) {
-                
-                mask = 1 << bit;
 
-                /*
-                 * if this bit is not present in the screens's enabled
-                 * display device mask (and the X screen has enabled
-                 * display devices), skip to the next bit
-                 */
+            /* Ignore display attributes, they are written later on */
 
-                if (((t->d & mask) == 0x0) && (t->d)) continue;
+            ret = nv_get_attribute_perms(h, a->attr, a->flags, &perms);
+            if (!ret || (perms.permissions & ATTRIBUTE_TYPE_DISPLAY)) {
+                continue;
+            }
 
-                status = NvCtrlGetValidDisplayAttributeValues
-                    (t->h, mask, a->attr, &valid);
+            /* Only write attributes that can be written */
 
-                if (status != NvCtrlSuccess) goto exit_bit_loop;
-                
-                if ((valid.permissions & ATTRIBUTE_TYPE_WRITE) == 0x0)
-                    goto exit_bit_loop;
-                
-                status = NvCtrlGetDisplayAttribute(t->h, mask, a->attr, &val);
-                
-                if (status != NvCtrlSuccess) goto exit_bit_loop;
-                
-                if (valid.permissions & ATTRIBUTE_TYPE_DISPLAY) {
+            status = NvCtrlGetValidAttributeValues(t->h, a->attr, &valid);
+            if (status != NvCtrlSuccess ||
+                !(valid.permissions & ATTRIBUTE_TYPE_WRITE) ||
+                (valid.permissions & ATTRIBUTE_TYPE_DISPLAY)) {;
+                continue;
+            }
 
-                    tmp_d_str =
-                        display_device_mask_to_display_device_name(mask);
+            status = NvCtrlGetAttribute(t->h, a->attr, &val);
+            if (status != NvCtrlSuccess) {
+                continue;
+            }
 
-                    fprintf(stream, "%s%c%s[%s]=%d\n", prefix,
-                            DISPLAY_NAME_SEPARATOR, a->name, tmp_d_str, val);
-                    
-                    free(tmp_d_str);
-                    
-                    continue;
-                    
-                } else {
-
-                    fprintf(stream, "%s%c%s=%d\n", prefix,
-                            DISPLAY_NAME_SEPARATOR, a->name, val);
-
-                    /* fall through to exit_bit_loop */
+            if (a->flags & NV_PARSER_TYPE_VALUE_IS_DISPLAY_ID) {
+                const char *name =
+                    nv_get_display_target_config_name(h, val);
+                if (name) {
+                    fprintf(stream, "%s%c%s=%s\n", prefix,
+                            DISPLAY_NAME_SEPARATOR, a->name, name);
                 }
-                
-            exit_bit_loop:
+                continue;
+            }
+
+            fprintf(stream, "%s%c%s=%d\n", prefix,
+                    DISPLAY_NAME_SEPARATOR, a->name, val);
 
-                bit = 25; /* XXX force us out of the display device loop */
-                
-            } /* bit */
-            
         } /* entry */
-        
+
     } /* screen */
+
+    /*
+     * Write attributes addressable to display targets
+     */
+
+    for (display = 0; display < h->targets[DISPLAY_TARGET].n; display++) {
+
+        t = &h->targets[DISPLAY_TARGET].t[display];
+
+        /* skip it if we don't have a handle for this display */
+
+        if (!t->h) continue;
+
+        /* 
+         * check to see if we have RANDR gamma available. We may
+         * skip writing attributes if it is missing. 
+         */
+
+        status = NvCtrlGetAttribute(t->h, 
+                                    NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE,
+                                    &randr_gamma_available);
+        if (status != NvCtrlSuccess) {
+            randr_gamma_available = 0;
+        }
+
+        /* Get the prefix we want to use for the display device target */
+
+        prefix = create_display_device_target_string(t, conf);
+
+        /* loop over all the entries in the table */
+
+        for (entry = 0; attributeTable[entry].name; entry++) {
+            const AttributeTableEntry *a = &attributeTable[entry];
+
+            /* 
+             * skip all attributes that are not supposed to be written
+             * to the config file
+             */
+
+            if (a->flags & NV_PARSER_TYPE_NO_CONFIG_WRITE) continue;
+
+            /* 
+             * for the display target we only write color attributes for now
+             */
+
+            if (a->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE) {
+                float c[3], b[3], g[3];
+
+                if (!randr_gamma_available) continue;
+
+                status = NvCtrlGetColorAttributes(t->h, c, b, g);
+                if (status != NvCtrlSuccess) continue;
+
+                fprintf(stream, "%s%c%s=%f\n",
+                        prefix, DISPLAY_NAME_SEPARATOR, a->name,
+                        get_color_value(a->attr, c, b, g));
+                continue;
+            }
+
+            /* Make sure this is a display attribute */
+
+            ret = nv_get_attribute_perms(h, a->attr, a->flags, &perms);
+            if (!ret || !(perms.permissions & ATTRIBUTE_TYPE_DISPLAY)) {
+                continue;
+            }
+
+            status = NvCtrlGetValidAttributeValues(t->h, a->attr, &valid);
+            if (status != NvCtrlSuccess ||
+                !(valid.permissions & ATTRIBUTE_TYPE_WRITE) ||
+                !(valid.permissions & ATTRIBUTE_TYPE_DISPLAY)) {
+                continue;
+            }
+
+            status = NvCtrlGetAttribute(t->h, a->attr, &val);
+            if (status == NvCtrlSuccess) {
+                fprintf(stream, "%s%c%s=%d\n", prefix,
+                        DISPLAY_NAME_SEPARATOR, a->name, val);
+            }
+        }
+
+        free(prefix);
+    }
     
     /*
      * loop the ParsedAttribute list, writing the attributes to file.
@@ -413,40 +488,32 @@ int nv_write_config_file(const char *filename, CtrlHandles *h,
          * target type other than an X screen, include a target
          * specification in what we write to the .rc file.
          */
-        
+
         target_str[0] = '\0';
-        
+
         if ((p->flags & NV_PARSER_HAS_TARGET) &&
             (p->target_type != NV_CTRL_TARGET_TYPE_X_SCREEN)) {
-            
-            int j;
-            
+
+            const TargetTypeEntry *targetTypeEntry;
+
             /* Find the target name of the target type */
-            for (j = 0; targetTypeTable[j].name; j++) {
-                if (targetTypeTable[j].nvctrl == p->target_type) {
-                    snprintf(target_str, 64, "[%s:%d]",
-                             targetTypeTable[j].parsed_name, p->target_id);
-                    break;
-                }
+            targetTypeEntry = nv_get_target_type_entry_by_nvctrl(p->target_type);
+            if (targetTypeEntry) {
+                snprintf(target_str, 64, "[%s:%d]",
+                         targetTypeEntry->parsed_name, p->target_id);
             }
         }
-        
-        if (p->display_device_mask) {
-            
-            tmp_d_str = display_device_mask_to_display_device_name
-                (p->display_device_mask);
-            
-            fprintf(stream, "%s%s%c%s[%s]=%d\n", p->display, target_str,
-                    DISPLAY_NAME_SEPARATOR, tmp, tmp_d_str, p->val);
-            
-            free(tmp_d_str);
-            
+
+        if (p->flags & NV_PARSER_TYPE_HIJACK_DISPLAY_DEVICE) {
+            fprintf(stream, "%s%s%c%s[0x%08x]=%d\n", p->display, target_str,
+                    DISPLAY_NAME_SEPARATOR, tmp, p->display_device_mask,
+                    p->val.i);
         } else {
-                
             fprintf(stream, "%s%s%c%s=%d\n", p->display, target_str,
-                    DISPLAY_NAME_SEPARATOR, tmp, p->val);
+                    DISPLAY_NAME_SEPARATOR, tmp, p->val.i);
         }
-        
+
+
         p = p->next;
     }
 
@@ -500,8 +567,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
         
         while (((c - buf) < length) &&
                (*c != '\n') &&
-               (*c != '\0') &&
-               (*c != EOF)) {
+               (*c != '\0')) {
             if (comment) { c++; continue; }
             if (*c == '#') { comment = c; continue; }
             if (!isspace(*c)) has_data = NV_TRUE;
@@ -519,7 +585,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
                 if (tmp) {
                     free(tmp);
                 }
-                tmp = malloc(sizeof(char) * current_tmp_len);
+                tmp = nvalloc(sizeof(char) * current_tmp_len);
             }
 
             strncpy (tmp, cur, len);
@@ -529,7 +595,7 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
 
             if (!parse_config_property(file, tmp, conf)) {
                 
-                w = realloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
+                w = nvrealloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
             
                 ret = nv_parse_attribute_string(tmp,
                                                 NV_PARSER_ASSIGNMENT,
@@ -546,15 +612,15 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
             }
         }
 
-        if (((c - buf) >= length) || (*c == '\0') || (*c == EOF)) cur = NULL;
+        if (((c - buf) >= length) || (*c == '\0')) cur = NULL;
         else cur = c + 1;
-        
+
         line++;
     }
     free(tmp);
     /* mark the end of the array */
 
-    w = realloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
+    w = nvrealloc(w, sizeof(ParsedAttributeWrapper) * (n+1));
     w[n].line = -1;
     
     return w;
@@ -577,11 +643,20 @@ static ParsedAttributeWrapper *parse_config_file(char *buf, const char *file,
 
 static int process_config_file_attributes(const char *file,
                                           ParsedAttributeWrapper *w,
-                                          const char *display_name)
+                                          const char *display_name,
+                                          CtrlHandlesArray *handles_array)
 {
-    int i, j, ret, found, n = 0;
-    CtrlHandles **h = NULL;
+    int i;
     
+    int old_verbosity = __verbosity;
+
+    /* Override the verbosity in the default behavior so
+     * nvidia-settings isn't so alarmist when loading the RC file.
+     */
+    if (!__verbosity_level_changed) {
+        __verbosity = VERBOSITY_NONE;
+    }
+
     /*
      * make sure that all ParsedAttributes have displays (this will do
      * nothing if we already have a display name
@@ -594,58 +669,31 @@ static int process_config_file_attributes(const char *file,
     /* build the list of CtrlHandles */
     
     for (i = 0; w[i].line != -1; i++) {
-        found = NV_FALSE;
-        for (j = 0; j < n; j++) {
-            if (nv_strcasecmp(h[j]->display, w[i].a.display)) {
-                w[i].h = h[j];
-                found = NV_TRUE;
-                break;
-            }
-        }
-
-        /*
-         * no handle found for this display, need to create a new
-         * handle.
-         *
-         * XXX we should really just build a list of what ctrl_handles
-         * we need, and what attributes on which ctrl_handles, so that
-         * we don't have to pass NV_CTRL_ATTRIBUTES_ALL_SUBSYSTEMS to
-         * NvCtrlAttributeInit (done in nv_alloc_ctrl_handles())
-         * unless we really need it.
-         */
-
-        if (!found) {
-            h = realloc(h, sizeof(CtrlHandles *) * (n + 1));
-            h[n] = nv_alloc_ctrl_handles(w[i].a.display);
-            w[i].h = h[n];
-            n++;
-        }
+        w[i].h = nv_alloc_ctrl_handles_and_add_to_array(w[i].a.display, 
+                                                        handles_array);
     }
     
     /* now process each attribute, passing in the correct CtrlHandles */
 
     for (i = 0; w[i].line != -1; i++) {
 
-        ret = nv_process_parsed_attribute(&w[i].a, w[i].h, NV_TRUE, NV_FALSE,
-                                          "on line %d of configuration file "
-                                          "'%s'", w[i].line, file);
+        nv_process_parsed_attribute(&w[i].a, w[i].h, NV_TRUE, NV_FALSE,
+                                    "on line %d of configuration file "
+                                    "'%s'", w[i].line, file);
         /*
          * We do not fail if processing the attribute failed.  If the
          * GPU or the X config changed (for example stereo is
          * disabled), some attributes written in the config file may
-         * not be advertised by the the NVCTRL extension (for example
-         * the control to force stereo)
+         * not be advertised by the NVCTRL extension (for example the
+         * control to force stereo)
          */
     }
     
-    /* free all the CtrlHandles we allocated */
+    /* Reset the default verbosity */
 
-    for (i = 0; i < n; i++) {
-        nv_free_ctrl_handles(h[i]);
+    if (!__verbosity_level_changed) {
+        __verbosity = old_verbosity;
     }
-    
-    if (h) free(h);
-
 
     return NV_TRUE;
     
@@ -711,6 +759,8 @@ ConfigPropertiesTableEntry configPropertyTable[] = {
     { "IncludeDisplayNameInConfigFile",
       CONFIG_PROPERTIES_INCLUDE_DISPLAY_NAME_IN_CONFIG_FILE },
     { "ShowQuitDialog", CONFIG_PROPERTIES_SHOW_QUIT_DIALOG },
+    { "UpdateRulesOnProfileNameChange",
+      CONFIG_PROPERTIES_UPDATE_RULES_ON_PROFILE_NAME_CHANGE },
     { NULL, 0 }
 };
 
@@ -758,23 +808,9 @@ static int parse_config_property(const char *file, const char *line, ConfigPrope
         if (!token)
             goto done;
 
-        c = malloc(sizeof(TimerConfigProperty));
-        if (!c) {
-            nv_warning_msg("Error parsing configuration file '%s': could "
-                           "not allocate memory for timer '%s'.",
-                           file, timer);
-            ret = NV_TRUE;
-            goto done;
-        }
+        c = nvalloc(sizeof(TimerConfigProperty));
 
         c->description = replace_characters(token, '_', ' ');
-        if (!c->description) {
-            nv_warning_msg("Error parsing configuration file '%s': could "
-                           "not allocate memory for timer '%s'.",
-                           file, timer);
-            ret = NV_TRUE;
-            goto done;
-        }
 
         token = strtok(NULL, ",");
         if (!token)
@@ -860,8 +896,6 @@ static void write_config_properties(FILE *stream, ConfigProperties *conf, char *
 
     for (c = conf->timers; (c != NULL); c = c->next) {
         description = replace_characters(c->description, ' ', '_');
-        if (!description)
-            continue;
         fprintf(stream, "Timer = %s,%s,%u\n",
                 description, c->user_enabled ? "Yes" : "No",
                 c->interval);
@@ -884,8 +918,67 @@ void init_config_properties(ConfigProperties *conf)
         (CONFIG_PROPERTIES_TOOLTIPS |
          CONFIG_PROPERTIES_DISPLAY_STATUS_BAR |
          CONFIG_PROPERTIES_SLIDER_TEXT_ENTRIES |
-         CONFIG_PROPERTIES_SHOW_QUIT_DIALOG);
+         CONFIG_PROPERTIES_SHOW_QUIT_DIALOG |
+         CONFIG_PROPERTIES_UPDATE_RULES_ON_PROFILE_NAME_CHANGE);
 
     conf->locale = strdup(setlocale(LC_NUMERIC, NULL));
 
 } /* init_config_properties() */
+
+
+/*
+ * create_display_device_target_string() - create the string
+ * to specify the display device target in the config file.
+ */
+
+static char *create_display_device_target_string(CtrlHandleTarget *t,
+                                                 ConfigProperties *conf)
+{
+    char *target_name = NULL;
+    char *target_prefix_name = NULL;
+    char *display_name = NULL;
+    char *s;
+    int target_type;
+
+    if (t->protoNames[NV_DPY_PROTO_NAME_RANDR]) {
+        target_name = t->protoNames[NV_DPY_PROTO_NAME_RANDR];
+    }
+
+    /* If we don't have a target name here, use the full name and return. */
+    if (!target_name) {
+        return nvstrdup(t->name);
+    }
+
+    /* Get the display name if the user requested it to be used. */
+    if (conf->booleans &
+        CONFIG_PROPERTIES_INCLUDE_DISPLAY_NAME_IN_CONFIG_FILE) {
+        display_name = NvCtrlGetDisplayName(t->h);
+    }
+
+    /* Get the target type prefix. */
+    target_type = NvCtrlGetTargetType(t->h);
+    if (target_type >= 0) {
+        target_prefix_name =
+            nvstrdup(targetTypeTable[target_type].parsed_name);
+        nvstrtoupper(target_prefix_name);
+    }
+
+    /* Build the string */
+
+    if (display_name && target_prefix_name) {
+        s = nvasprintf("%s[%s:%s]", display_name,
+                       target_prefix_name, target_name);
+    } else if (target_prefix_name) {
+        s = nvasprintf("[%s:%s]", target_prefix_name, target_name);
+    } else if (display_name) {
+        s = nvasprintf("%s[%s]", display_name, target_name);
+    } else {
+        s = nvasprintf("[%s]", target_name);
+    }
+
+    free(target_prefix_name);
+    free(display_name);
+
+    return s;
+}
+
