@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2010 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -37,8 +32,14 @@
 #include "ctkconfig.h"
 #include "ctkhelp.h"
 #include "ctkcolorcontrols.h"
+#include "ctkdropdownmenu.h"
 
 /* function prototypes */
+static void
+ctk_color_controls_class_init(CtkColorControlsClass *ctk_object_class);
+
+static void ctk_color_controls_finalize(GObject *object);
+
 static gboolean build_color_space_table(CtkColorControls *ctk_color_controls,
                                         NVCTRLAttributeValidValuesRec valid);
 
@@ -48,9 +49,11 @@ static gint map_nvctrl_value_to_table(CtkColorControls *ctk_color_controls,
 static
 gboolean update_color_space_menu_info(CtkColorControls *ctk_color_controls);
 
-static void color_space_menu_changed(GtkOptionMenu *color_space_menu,
+static void setup_reset_button(CtkColorControls *ctk_color_controls);
+
+static void color_space_menu_changed(GtkWidget *widget,
                                      gpointer user_data);
-static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
+static void color_range_menu_changed(GtkWidget *widget,
                                      gpointer user_data);
 
 static void color_control_update_received(GtkObject *object, gpointer arg1,
@@ -70,10 +73,15 @@ void post_color_space_update(CtkColorControls *ctk_color_controls,
 /* help text */
 static const char * __color_controls_help =
 "The Color Controls allow changing the color space and color range "
-"of the display device.  The possible values for Color Space vary "
-"depending on the capabilities of the display device and the GPU, but "
-"may contain \"RGB\", \"YCbCr422\", and \"YCbCr444\".  The possible values "
-"for Color Range are \"Limited\" and \"Full\".";
+"of the display device.";
+
+static const char * __color_space_help =
+"The possible values for Color Space vary depending on the capabilities of "
+"the display device and the GPU, but may contain \"RGB\", \"YCbCr422\", "
+"and \"YCbCr444\".";  
+
+static const char * __color_range_help =
+"The possible values for Color Range are \"Limited\" and \"Full\".";
 
 GType ctk_color_controls_get_type(void)
 {
@@ -84,12 +92,13 @@ GType ctk_color_controls_get_type(void)
             sizeof (CtkColorControlsClass),
             NULL, /* base_init */
             NULL, /* base_finalize */
-            NULL, /* class_init, */
+            (GClassInitFunc) ctk_color_controls_class_init, /* class_init, */
             NULL, /* class_finalize */
             NULL, /* class_data */
             sizeof (CtkColorControls),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_color_controls_type =
@@ -101,28 +110,54 @@ GType ctk_color_controls_get_type(void)
     return ctk_color_controls_type;
 } /* ctk_color_controls_get_type() */
 
+
+
+static void
+ctk_color_controls_class_init(CtkColorControlsClass *ctk_object_class)
+{
+    GObjectClass *gobject_class = (GObjectClass *)ctk_object_class;
+    gobject_class->finalize = ctk_color_controls_finalize;
+}
+
+
+
+static void ctk_color_controls_finalize(GObject *object)
+{
+    CtkColorControls *ctk_object = CTK_COLOR_CONTROLS(object);
+
+    g_signal_handlers_disconnect_matched(G_OBJECT(ctk_object->ctk_event),
+                                         G_SIGNAL_MATCH_DATA,
+                                         0,
+                                         0,
+                                         NULL,
+                                         NULL,
+                                         (gpointer) ctk_object);
+}
+
+
+
 GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
                                   CtkConfig *ctk_config,
                                   CtkEvent *ctk_event,
                                   GtkWidget *reset_button,
-                                  unsigned int display_device_mask,
                                   char *name)
 {
     GObject *object;
     CtkColorControls *ctk_color_controls;
-    GtkWidget *frame, *hbox, *label, *eventbox;
-    GtkWidget *menu, *table, *menu_item = NULL, *separator;
+    GtkWidget *frame, *hbox, *label;
+    GtkWidget *table, *separator;
+    CtkDropDownMenu *menu;
     ReturnStatus ret1, ret2;
     NVCTRLAttributeValidValuesRec valid1, valid2;
     gint i;
 
     /* check if color configuration is supported */
-    ret1 = NvCtrlGetValidDisplayAttributeValues(handle, display_device_mask,
-                                                NV_CTRL_COLOR_SPACE,
-                                                &valid1);
-    ret2 = NvCtrlGetValidDisplayAttributeValues(handle, display_device_mask,
-                                                NV_CTRL_COLOR_RANGE,
-                                                &valid2);
+    ret1 = NvCtrlGetValidAttributeValues(handle,
+                                         NV_CTRL_COLOR_SPACE,
+                                         &valid1);
+    ret2 = NvCtrlGetValidAttributeValues(handle,
+                                         NV_CTRL_COLOR_RANGE,
+                                         &valid2);
 
     if ((ret1 != NvCtrlSuccess) || (ret2 != NvCtrlSuccess)) {
         return NULL;
@@ -137,8 +172,8 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     ctk_color_controls = CTK_COLOR_CONTROLS(object);
     ctk_color_controls->handle = handle;
     ctk_color_controls->ctk_config = ctk_config;
+    ctk_color_controls->ctk_event = ctk_event;
     ctk_color_controls->reset_button = reset_button;
-    ctk_color_controls->display_device_mask = display_device_mask;
     ctk_color_controls->name = strdup(name);
 
     /* build a table holding available color space */
@@ -150,13 +185,10 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, FRAME_PADDING);
-    ctk_color_controls->color_controls_main = hbox;
+    ctk_color_controls->color_controls_box = hbox;
 
     frame = gtk_frame_new("Color Controls");
-    eventbox = gtk_event_box_new();
-    gtk_container_add(GTK_CONTAINER(eventbox), frame);
-    gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
-    ctk_config_set_tooltip(ctk_config, eventbox, __color_controls_help);
+    gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
     table = gtk_table_new(1, 6, FALSE);
     gtk_container_add(GTK_CONTAINER(frame), table);
@@ -165,29 +197,27 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 
     /* dropdown list for color space */
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
 
     for (i = 0; i < ctk_color_controls->color_space_table_size; i++) {
         switch (ctk_color_controls->color_space_table[i]) {
         case NV_CTRL_COLOR_SPACE_YCbCr422:
-            menu_item = gtk_menu_item_new_with_label("YCbCr422");
+            ctk_drop_down_menu_append_item(menu, "YCbCr422", i);
             break;
         case NV_CTRL_COLOR_SPACE_YCbCr444:
-            menu_item = gtk_menu_item_new_with_label("YCbCr444");
+            ctk_drop_down_menu_append_item(menu, "YCbCr444", i);
             break;
         default:
         case NV_CTRL_COLOR_SPACE_RGB:
-            menu_item = gtk_menu_item_new_with_label("RGB");
+            ctk_drop_down_menu_append_item(menu, "RGB", i);
             break;
         }
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
     }
-
-    ctk_color_controls->color_space_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         menu);
+    ctk_color_controls->color_space_menu = GTK_WIDGET(menu);
+    ctk_config_set_tooltip(ctk_config, 
+                           ctk_color_controls->color_space_menu,
+                           __color_space_help);
 
     /* If dropdown only has one item, disable it */
     if (ctk_color_controls->color_space_table_size > 1) {
@@ -210,7 +240,6 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
     hbox = gtk_hbox_new(FALSE, 0);
-    ctk_color_controls->color_space_box = hbox;
     gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_box_pack_start(GTK_BOX(hbox),
@@ -226,8 +255,13 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 
     /* Build color widgets & pack them in table */
     /* dropdown list for color range */
-
-    ctk_color_controls->color_range_menu = gtk_option_menu_new();
+    
+    ctk_color_controls->color_range_menu =
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+    
+    ctk_config_set_tooltip(ctk_config, 
+                           ctk_color_controls->color_range_menu,
+                           __color_range_help);
 
     g_signal_connect(G_OBJECT(ctk_color_controls->color_range_menu),
                      "changed", G_CALLBACK(color_range_menu_changed),
@@ -267,6 +301,53 @@ GtkWidget* ctk_color_controls_new(NvCtrlAttributeHandle *handle,
 } /* ctk_color_controls_new() */
 
 
+
+/*
+ * setup_reset_button() - enables the reset button if any of the current
+ * settings are not the default.
+ */
+static void setup_reset_button(CtkColorControls *ctk_color_controls)
+{
+    gint history;
+    gint val;
+    CtkDropDownMenu *color_space_menu, *color_range_menu;
+
+    if (!GTK_WIDGET_SENSITIVE(ctk_color_controls->color_controls_box)) {
+        /* Nothing is available, don't bother enabling the reset button yet. */
+        return;
+    }
+
+    /* The color space menu is always available */
+    color_space_menu = CTK_DROP_DOWN_MENU(ctk_color_controls->color_space_menu);
+    history = color_space_menu->current_selected_item;
+
+    val = ctk_color_controls->color_space_table[history];
+    if (val != NV_CTRL_COLOR_SPACE_RGB) {
+        goto enable;
+    }
+
+    /* Color range is dependent on the color space */
+    if (GTK_WIDGET_SENSITIVE(ctk_color_controls->color_range_menu)) {
+        color_range_menu = 
+            CTK_DROP_DOWN_MENU(ctk_color_controls->color_range_menu);
+        history = color_range_menu->current_selected_item; 
+        val = ctk_color_controls->color_range_table[history];
+        if (val != NV_CTRL_COLOR_RANGE_FULL) {
+            goto enable;
+        }
+    }
+
+    /* Don't disable reset button here, since other settings that are not
+     * managed by the ctk_image_slider here may need it enabled
+     */
+    return;
+
+ enable:
+    gtk_widget_set_sensitive(ctk_color_controls->reset_button, TRUE);
+}
+
+
+
 /*
  * ctk_color_controls_setup() - Setup routine for color attributes. Used
  * in DFP setup stage as well as for updating the GUI when there is change in
@@ -280,10 +361,12 @@ void ctk_color_controls_setup(CtkColorControls *ctk_color_controls)
 
     /* color space */
     if (!update_color_space_menu_info(ctk_color_controls)) {
-        gtk_widget_set_sensitive(ctk_color_controls->color_controls_main,
-                                 FALSE);
-        gtk_widget_hide_all(ctk_color_controls->color_controls_main);
+        gtk_widget_set_sensitive(ctk_color_controls->color_controls_box, FALSE);
+        gtk_widget_hide_all(ctk_color_controls->color_controls_box);
     }
+
+    setup_reset_button(ctk_color_controls);
+
 } /* ctk_color_controls_setup() */
 
 
@@ -293,11 +376,9 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
 
     /* color space */
     if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_color_controls->handle,
-                                  ctk_color_controls->display_device_mask,
-                                  NV_CTRL_COLOR_SPACE,
-                                  &color_space)) {
-        free(ctk_color_controls->color_space_table);
+        NvCtrlGetAttribute(ctk_color_controls->handle,
+                           NV_CTRL_COLOR_SPACE,
+                           &color_space)) {
         return FALSE;
     }
 
@@ -309,9 +390,8 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
          G_CALLBACK(color_space_menu_changed),
          (gpointer) ctk_color_controls);
 
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         color_space);
+    ctk_drop_down_menu_set_current_value
+        (CTK_DROP_DOWN_MENU(ctk_color_controls->color_space_menu), color_space);
 
     g_signal_handlers_unblock_by_func
         (G_OBJECT(ctk_color_controls->color_space_menu),
@@ -319,9 +399,14 @@ static gboolean update_color_space_menu_info(CtkColorControls *ctk_color_control
          (gpointer) ctk_color_controls);
 
     /* dynamically regenerate color range dropdown */
-    setup_color_range_dropdown(ctk_color_controls);
+    if (!setup_color_range_dropdown(ctk_color_controls)) {
+        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, FALSE);
+    } else {
+        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, TRUE);
+    }
+
+    return TRUE;
 
-    return True;
 } /* update_color_space_menu_info() */
 
 static
@@ -357,34 +442,20 @@ void post_color_space_update(CtkColorControls *ctk_color_controls,
                                  ctk_color_controls->name);
 }
 
-static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
+static void color_range_menu_changed(GtkWidget *widget,
                                      gpointer user_data)
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_range = NV_CTRL_COLOR_RANGE_FULL;
 
-    history = gtk_option_menu_get_history(color_range_menu);
+    history = ctk_drop_down_menu_get_current_value(menu);
     color_range = ctk_color_controls->color_range_table[history];
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_RANGE,
-                              color_range);
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         color_range);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_RANGE,
+                       color_range);
 
     /* reflecting the change to statusbar message and the reset button */
     post_color_range_update(ctk_color_controls, color_range);
@@ -392,37 +463,24 @@ static void color_range_menu_changed(GtkOptionMenu *color_range_menu,
 } /* color_range_menu_changed() */
 
 
-static void color_space_menu_changed(GtkOptionMenu *color_space_menu,
+static void color_space_menu_changed(GtkWidget *widget,
                                      gpointer user_data)
 {
     CtkColorControls *ctk_color_controls =
         CTK_COLOR_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
     gint history, color_space = NV_CTRL_COLOR_SPACE_RGB;
-
-    history = gtk_option_menu_get_history(color_space_menu);
+    
+    history = ctk_drop_down_menu_get_current_value(menu);
 
     color_space = ctk_color_controls->color_space_table[history];
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_SPACE,
-                              color_space);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_SPACE,
+                       color_space);
 
     color_space = map_nvctrl_value_to_table(ctk_color_controls,
                                             color_space);
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_space_menu),
-         G_CALLBACK(color_space_menu_changed),
-         (gpointer) ctk_color_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_space_menu),
-         color_space);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_color_controls->color_space_menu),
-         G_CALLBACK(color_space_menu_changed),
-         (gpointer) ctk_color_controls);
 
     /* reflecting the change in color space to other widgets & reset button */
     ctk_color_controls_setup(ctk_color_controls);
@@ -441,15 +499,13 @@ void ctk_color_controls_reset(CtkColorControls *ctk_color_controls)
         return;
     }
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_SPACE,
-                              NV_CTRL_COLOR_SPACE_RGB);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_SPACE,
+                       NV_CTRL_COLOR_SPACE_RGB);
 
-    NvCtrlSetDisplayAttribute(ctk_color_controls->handle,
-                              ctk_color_controls->display_device_mask,
-                              NV_CTRL_COLOR_RANGE,
-                              NV_CTRL_COLOR_RANGE_FULL);
+    NvCtrlSetAttribute(ctk_color_controls->handle,
+                       NV_CTRL_COLOR_RANGE,
+                       NV_CTRL_COLOR_RANGE_FULL);
 
     ctk_color_controls_setup(ctk_color_controls);
 } /* ctk_color_controls_reset() */
@@ -462,8 +518,18 @@ void add_color_controls_help(CtkColorControls *ctk_color_controls,
                              GtkTextBuffer *b,
                              GtkTextIter *i)
 {
+    if (!ctk_color_controls) {
+        return;
+    }
+
     ctk_help_heading(b, i, "Color Controls");
-    ctk_help_para(b, i, __color_controls_help);
+    ctk_help_para(b, i, "%s", __color_controls_help);
+
+    ctk_help_term(b, i, "Color Space");
+    ctk_help_para(b, i, "%s", __color_space_help);
+
+    ctk_help_term(b, i, "Color Range");
+    ctk_help_para(b, i, "%s", __color_range_help);
 } /* add_color_controls_help() */
 
 
@@ -478,14 +544,8 @@ static void color_control_update_received(GtkObject *object, gpointer arg1,
     CtkColorControls *ctk_object = CTK_COLOR_CONTROLS(user_data);
     CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
 
-    /* if the event is not for this display device, return */
-
-    if (!(event_struct->display_mask & ctk_object->display_device_mask)) {
-        return;
-    }
-
     ctk_color_controls_setup(ctk_object);
-    
+
     /* update status bar message */
     switch (event_struct->attribute) {
     case NV_CTRL_COLOR_RANGE:
@@ -545,13 +605,15 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
     gint i, n = 0, color_range_count = 0;
     gint mask, val;
     ReturnStatus ret;
-    GtkWidget *menu, *menu_item;
     NVCTRLAttributeValidValuesRec valid;
+    CtkDropDownMenu *d;
 
-    ret = NvCtrlGetValidDisplayAttributeValues(ctk_color_controls->handle,
-                                               ctk_color_controls->display_device_mask,
-                                               NV_CTRL_COLOR_RANGE,
-                                               &valid);
+    ret = NvCtrlGetValidAttributeValues(ctk_color_controls->handle,
+                                        NV_CTRL_COLOR_RANGE,
+                                        &valid);
+    if (ret != NvCtrlSuccess) {
+        return FALSE;
+    }
 
     if (valid.type != ATTRIBUTE_TYPE_INT_BITS) {
         return FALSE;
@@ -582,55 +644,47 @@ static gboolean setup_color_range_dropdown(CtkColorControls *ctk_color_controls)
     }
     
     /* dropdown list for color range */
-    menu = gtk_menu_new();
+    d = (CtkDropDownMenu *) ctk_color_controls->color_range_menu;
+
+    g_signal_handlers_block_by_func
+         (G_OBJECT(ctk_color_controls->color_range_menu),
+         G_CALLBACK(color_range_menu_changed),
+         (gpointer) ctk_color_controls);
+    
+    ctk_drop_down_menu_reset(d);
 
     for (i = 0; i < ctk_color_controls->color_range_table_size; i++) {
         switch (ctk_color_controls->color_range_table[i]) {
         case NV_CTRL_COLOR_RANGE_FULL:
-            menu_item = gtk_menu_item_new_with_label("Full");
+            ctk_drop_down_menu_append_item(d, "Full", i);
             break;
         default:
         case NV_CTRL_COLOR_RANGE_LIMITED:
-            menu_item = gtk_menu_item_new_with_label("Limited");
+            ctk_drop_down_menu_append_item(d, "Limited", i);
             break;
         }
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
     }
 
     /* color range */
     if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_color_controls->handle,
-                                  ctk_color_controls->display_device_mask,
-                                  NV_CTRL_COLOR_RANGE, &val)) {
+        NvCtrlGetAttribute(ctk_color_controls->handle,
+                           NV_CTRL_COLOR_RANGE,
+                           &val)) {
         val = NV_CTRL_COLOR_RANGE_FULL;
     }
 
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_color_controls->color_range_menu),
-         G_CALLBACK(color_range_menu_changed),
-         (gpointer) ctk_color_controls);
+    ctk_drop_down_menu_set_current_value(d, val);
 
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         menu);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_color_controls->color_range_menu),
-         val);
-
-    /* If dropdown only has one item, disable it */
-    if (ctk_color_controls->color_range_table_size > 1) {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, True);
-    } else {
-        gtk_widget_set_sensitive(ctk_color_controls->color_range_menu, False);
-    }
-    
     g_signal_handlers_unblock_by_func
         (G_OBJECT(ctk_color_controls->color_range_menu),
          G_CALLBACK(color_range_menu_changed),
          (gpointer) ctk_color_controls);
 
+    /* If dropdown only has one item, disable it */
+    if (ctk_color_controls->color_range_table_size <= 1) {
+        return FALSE;
+    }
+
     return TRUE;
 
 } /* setup_color_range_dropdown() */
