@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -30,7 +25,6 @@
 #include "red_xpm.h"
 #include "green_xpm.h"
 #include "blue_xpm.h"
-#include "ctkbanner.h"
 
 #include "ctkcurve.h"
 #include "ctkscale.h"
@@ -59,7 +53,7 @@ static const char *__confirm_button_help = "Some color correction settings "
 "button by then to accept the changes, it will restore your previous values.";
 
 static const char *__color_curve_help = "The color curve graph changes to "
-"reflect your adjustments made with the Brightness, Constrast, and Gamma "
+"reflect your adjustments made with the Brightness, Contrast, and Gamma "
 "sliders.";
 
 static void
@@ -148,6 +142,7 @@ GType ctk_color_correction_get_type(
             sizeof (CtkColorCorrection),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_color_correction_type =
@@ -163,10 +158,6 @@ static void
 ctk_color_correction_class_init(CtkColorCorrectionClass
                                 *ctk_color_correction_class)
 {
-    GObjectClass *gobject_class;
-
-    gobject_class = (GObjectClass *) ctk_color_correction_class;
-
     signals[CHANGED] =
         g_signal_new("changed",
                      G_OBJECT_CLASS_TYPE(ctk_color_correction_class),
@@ -175,8 +166,6 @@ ctk_color_correction_class_init(CtkColorCorrectionClass
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
-
-
 }
 
 
@@ -190,7 +179,6 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
 
     GtkWidget *menu;
     GtkWidget *image;
-    GtkWidget *banner;
     GtkWidget *label;
     GtkWidget *scale;
     GtkWidget *curve;
@@ -205,20 +193,7 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     GtkWidget *widget;
     GtkWidget *hsep;
     GtkWidget *eventbox;
-    ReturnStatus ret;
-    gint val;
 
-    /* check if the VidMode extension is present */
-
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_ATTR_EXT_VM_PRESENT, &val);
-    if ((ret != NvCtrlSuccess) || (val == FALSE)) return NULL;
-    
-    /* check if the noScanout mode enabled */
-
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_NO_SCANOUT, &val);
-    if ((ret == NvCtrlSuccess) && (val == NV_CTRL_NO_SCANOUT_ENABLED))
-        return NULL;
-    
     object = g_object_new(CTK_TYPE_COLOR_CORRECTION, NULL);
 
     ctk_color_correction = CTK_COLOR_CORRECTION(object);
@@ -231,16 +206,6 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
 
     gtk_box_set_spacing(GTK_BOX(ctk_color_correction), 10);
 
-    /*
-     * Banner: TOP - LEFT -> RIGHT
-     *
-     * This image serves as a visual reference for basic color_box correction
-     * purposes.
-     */
-
-    banner = ctk_banner_image_new(BANNER_ARTWORK_COLOR);
-    gtk_box_pack_start(GTK_BOX(ctk_color_correction), banner, FALSE, FALSE, 0);
-
     /* create the main hbox and the two main vboxes*/
 
     mainhbox = gtk_hbox_new(FALSE, 10);
@@ -892,71 +857,101 @@ static void apply_parsed_attribute_list(
     target_id = NvCtrlGetTargetId(ctk_color_correction->handle);
 
     while (p) {
+        CtrlHandleTargetNode *node;
 
         if (!p->next) goto next_attribute;
         
         if (!(p->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE)) goto next_attribute;
         
         /*
-         * if this parsed attribute's target_type, target_id does not match the
-         * current target_type and target_id then ignore
+         * Apply the parsed attribute's settings only if the color 
+         * correction's target matches one of the (parse attribute's)
+         * specification targets.
          */
         
-        if ((p->target_type != target_type) ||
-            (p->target_id != target_id)) goto next_attribute;
+        for (node = p->targets; node && node->next; node = node->next) {
+
+            int attr_target_type = NvCtrlGetTargetType(node->t->h);
+            int attr_target_id = NvCtrlGetTargetId(node->t->h);
+
+            if ((attr_target_type != target_type) ||
+                (attr_target_id != target_id)) {
+                continue;
+            }
+            
+            switch (p->attr & (ALL_VALUES | ALL_CHANNELS)) {
+            case (CONTRAST_VALUE | RED_CHANNEL):
+                set_color_state(ctk_color_correction, CONTRAST,
+                                RED_CHANNEL, p->val.f, TRUE); break;
+            case (CONTRAST_VALUE | GREEN_CHANNEL):
+                set_color_state(ctk_color_correction, CONTRAST,
+                                GREEN_CHANNEL, p->val.f, TRUE); break;
+            case (CONTRAST_VALUE | BLUE_CHANNEL):
+                set_color_state(ctk_color_correction, CONTRAST,
+                                BLUE_CHANNEL, p->val.f, TRUE); break;
+            case (CONTRAST_VALUE | ALL_CHANNELS):
+                set_color_state(ctk_color_correction, CONTRAST,
+                                ALL_CHANNELS, p->val.f, TRUE); break;
+
+            case (BRIGHTNESS_VALUE | RED_CHANNEL):
+                set_color_state(ctk_color_correction, BRIGHTNESS,
+                                RED_CHANNEL, p->val.f, TRUE); break;
+            case (BRIGHTNESS_VALUE | GREEN_CHANNEL):
+                set_color_state(ctk_color_correction, BRIGHTNESS,
+                                GREEN_CHANNEL, p->val.f, TRUE); break;
+            case (BRIGHTNESS_VALUE | BLUE_CHANNEL):
+                set_color_state(ctk_color_correction, BRIGHTNESS,
+                                BLUE_CHANNEL, p->val.f, TRUE); break;
+            case (BRIGHTNESS_VALUE | ALL_CHANNELS):
+                set_color_state(ctk_color_correction, BRIGHTNESS,
+                                ALL_CHANNELS, p->val.f, TRUE); break;
+
+            case (GAMMA_VALUE | RED_CHANNEL):
+                set_color_state(ctk_color_correction, GAMMA,
+                                RED_CHANNEL, p->val.f, TRUE); break;
+            case (GAMMA_VALUE | GREEN_CHANNEL):
+                set_color_state(ctk_color_correction, GAMMA,
+                                GREEN_CHANNEL, p->val.f, TRUE); break;
+            case (GAMMA_VALUE | BLUE_CHANNEL):
+                set_color_state(ctk_color_correction, GAMMA,
+                                BLUE_CHANNEL, p->val.f, TRUE); break;
+            case (GAMMA_VALUE | ALL_CHANNELS):
+                set_color_state(ctk_color_correction, GAMMA,
+                                ALL_CHANNELS, p->val.f, TRUE); break;
+
+            default:
+                continue;
+            }
         
-        switch (p->attr & (ALL_VALUES | ALL_CHANNELS)) {
-        case (CONTRAST_VALUE | RED_CHANNEL):
-            set_color_state(ctk_color_correction, CONTRAST,
-                            RED_CHANNEL, p->fval, TRUE); break;
-        case (CONTRAST_VALUE | GREEN_CHANNEL):
-            set_color_state(ctk_color_correction, CONTRAST,
-                            GREEN_CHANNEL, p->fval, TRUE); break;
-        case (CONTRAST_VALUE | BLUE_CHANNEL):
-            set_color_state(ctk_color_correction, CONTRAST,
-                            BLUE_CHANNEL, p->fval, TRUE); break;
-        case (CONTRAST_VALUE | ALL_CHANNELS):
-            set_color_state(ctk_color_correction, CONTRAST,
-                            ALL_CHANNELS, p->fval, TRUE); break;
+            attr |= (p->attr & (ALL_VALUES | ALL_CHANNELS));
 
-        case (BRIGHTNESS_VALUE | RED_CHANNEL):
-            set_color_state(ctk_color_correction, BRIGHTNESS,
-                            RED_CHANNEL, p->fval, TRUE); break;
-        case (BRIGHTNESS_VALUE | GREEN_CHANNEL):
-            set_color_state(ctk_color_correction, BRIGHTNESS,
-                            GREEN_CHANNEL, p->fval, TRUE); break;
-        case (BRIGHTNESS_VALUE | BLUE_CHANNEL):
-            set_color_state(ctk_color_correction, BRIGHTNESS,
-                            BLUE_CHANNEL, p->fval, TRUE); break;
-        case (BRIGHTNESS_VALUE | ALL_CHANNELS):
-            set_color_state(ctk_color_correction, BRIGHTNESS,
-                            ALL_CHANNELS, p->fval, TRUE); break;
-
-        case (GAMMA_VALUE | RED_CHANNEL):
-            set_color_state(ctk_color_correction, GAMMA,
-                            RED_CHANNEL, p->fval, TRUE); break;
-        case (GAMMA_VALUE | GREEN_CHANNEL):
-            set_color_state(ctk_color_correction, GAMMA,
-                            GREEN_CHANNEL, p->fval, TRUE); break;
-        case (GAMMA_VALUE | BLUE_CHANNEL):
-            set_color_state(ctk_color_correction, GAMMA,
-                            BLUE_CHANNEL, p->fval, TRUE); break;
-        case (GAMMA_VALUE | ALL_CHANNELS):
-            set_color_state(ctk_color_correction, GAMMA,
-                            ALL_CHANNELS, p->fval, TRUE); break;
-
-        default:
-            goto next_attribute;
         }
         
-        attr |= (p->attr & (ALL_VALUES | ALL_CHANNELS));
-        
     next_attribute:
         
         p = p->next;
     }
 
     if (attr) {
+
+        int i;
+
+        /*
+         * if all the separate color channels are the same for an
+         * attribute, propagate the value to ALL_CHANNELS for that
+         * attribute
+         */
+        for (i = CONTRAST; i <= GAMMA; i++) {
+            float val = ctk_color_correction->cur_slider_val[i][RED];
+
+            if ((ctk_color_correction->cur_slider_val[i][GREEN] == val) &&
+                (ctk_color_correction->cur_slider_val[i][BLUE] == val)) {
+                set_color_state(ctk_color_correction, i,
+                                ALL_CHANNELS, val, TRUE);
+                attr |= ALL_CHANNELS;
+            }
+        }
+
         NvCtrlSetColorAttributes(ctk_color_correction->handle,
                                  ctk_color_correction->cur_slider_val[CONTRAST],
                                  ctk_color_correction->cur_slider_val[BRIGHTNESS],
@@ -968,7 +963,7 @@ static void apply_parsed_attribute_list(
 
 /** update_confirm_text() ************************************
  *
- * Generates the text used to lable confirmation button.
+ * Generates the text used to label confirmation button.
  *
  **/
 
@@ -1047,18 +1042,36 @@ GtkTextBuffer *ctk_color_correction_create_help(GtkTextTagTable *table)
 {
     GtkTextIter i;
     GtkTextBuffer *b;
-
+    const gchar *title = "X Server Color Correction";
+    
     b = gtk_text_buffer_new(table);
     
     gtk_text_buffer_get_iter_at_offset(b, &i, 0);
     
-    ctk_help_title(b, &i, "X Server Color Correction Help");
+    ctk_help_title(b, &i, "%s Help", title);
 
-    ctk_help_heading(b, &i, "Active Color Channel");
-    ctk_help_para(b, &i, __active_color_help);
+    ctk_color_correction_tab_help(b, &i, title, FALSE /* randr */);
+    
+    ctk_help_heading(b, &i, "Reset Hardware Defaults");
+    ctk_help_para(b, &i, "%s", __resest_button_help);
 
-    ctk_help_heading(b, &i, "Brightness, Contrast and Gamma");
-    ctk_help_para(b, &i, "The Brightness, Contrast and Gamma sliders "
+    ctk_help_finish(b);
+
+    return b;
+}
+
+
+void ctk_color_correction_tab_help(GtkTextBuffer *b, GtkTextIter *i,
+                                   const gchar *title,
+                                   gboolean randr)
+{
+    ctk_help_heading(b, i, "Color Correction");
+
+    ctk_help_term(b, i, "Active Color Channel");
+    ctk_help_para(b, i, "%s", __active_color_help);
+
+    ctk_help_term(b, i, "Brightness, Contrast and Gamma");
+    ctk_help_para(b, i, "The Brightness, Contrast and Gamma sliders "
                   "allow you to adjust the brightness, contrast, "
                   "or gamma values for the selected color channel(s).  This "
                   "helps you to compensate "
@@ -1069,24 +1082,21 @@ GtkTextBuffer *ctk_color_correction_create_help(GtkTextTagTable *table)
                   "as photographs) when they are displayed on your "
                   "monitor.");
 
-    ctk_help_para(b, &i, "Also, many 3D-accelerated games may appear too "
+    ctk_help_para(b, i, "Also, many 3D-accelerated games may appear too "
                   "dark to play.  Increasing the brightness and/or gamma "
                   "value equally across all channels will make these games "
                   "appear brighter, making them more playable.");
-    
-    ctk_help_para(b, &i, __color_curve_help);
 
-    ctk_help_para(b, &i, "Note that the X Server Color Correction page uses "
-                  "the XF86VidMode extension to manipulate the X screen's "
-                  "color ramps.");
-    
-    ctk_help_heading(b, &i, "Confirm Current Changes");
-    ctk_help_para(b, &i, __confirm_button_help);
-    
-    ctk_help_heading(b, &i, "Reset Hardware Defaults");
-    ctk_help_para(b, &i, __resest_button_help);
+    ctk_help_para(b, i, "%s", __color_curve_help);
 
-    ctk_help_finish(b);
+    if (randr) {
+        ctk_help_para(b, i, "The %s tab uses the RandR extension to "
+                      "manipulate an RandR CRTC's gamma ramp.", title);
+    } else {
+        ctk_help_para(b, i, "The %s page uses the XF86VidMode extension "
+                      "to manipulate the X screen's gamma ramps", title);
+    }
 
-    return b;
+    ctk_help_term(b, i, "Confirm Current Changes");
+    ctk_help_para(b, i, "%s", __confirm_button_help);
 }
