@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 /*
@@ -33,6 +28,8 @@
  *   NV-CONTROL -> event -> glib -> CtkEvent -> signal -> GUI
  */
 
+#include <string.h>
+
 #include <gtk/gtk.h>
 
 #include <X11/Xlib.h> /* Xrandr */
@@ -94,6 +91,7 @@ GType ctk_event_get_type(void)
             sizeof(CtkEvent),
             0,                  /* n_preallocs */
             NULL,               /* instance_init */
+            NULL                /* value_table */
         };
 
         ctk_event_type = g_type_register_static
@@ -107,11 +105,8 @@ GType ctk_event_get_type(void)
 
 static void ctk_event_class_init(CtkEventClass *ctk_event_class)
 {
-    GObjectClass *gobject_class;
     gint i;
 
-    gobject_class = (GObjectClass *) ctk_event_class;
-
     /* clear the signal array */
 
     for (i = 0; i <= NV_CTRL_LAST_ATTRIBUTE; i++) signals[i] = 0;
@@ -162,13 +157,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_FORCE_STEREO);
     MAKE_SIGNAL(NV_CTRL_ARCHITECTURE);
     MAKE_SIGNAL(NV_CTRL_TEXTURE_CLAMPING);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_ALPHA);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_RED);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_GREEN);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_BLUE);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_X_OFFSET);
-    MAKE_SIGNAL(NV_CTRL_CURSOR_SHADOW_Y_OFFSET);
     MAKE_SIGNAL(NV_CTRL_FSAA_APPLICATION_CONTROLLED);
     MAKE_SIGNAL(NV_CTRL_LOG_ANISO_APPLICATION_CONTROLLED);
     MAKE_SIGNAL(NV_CTRL_IMAGE_SHARPENING);
@@ -190,7 +178,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_GVIO_REQUESTED_VIDEO_FORMAT);
     MAKE_SIGNAL(NV_CTRL_GVIO_DETECTED_VIDEO_FORMAT);
     MAKE_SIGNAL(NV_CTRL_GVO_DATA_FORMAT);
-    MAKE_SIGNAL(NV_CTRL_GVO_DISPLAY_X_SCREEN);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECTED);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_SYNC_INPUT_DETECT_MODE);
     MAKE_SIGNAL(NV_CTRL_GVO_SDI_SYNC_INPUT_DETECTED);
@@ -203,18 +190,17 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_GVIO_VIDEO_FORMAT_WIDTH);
     MAKE_SIGNAL(NV_CTRL_GVIO_VIDEO_FORMAT_HEIGHT);
     MAKE_SIGNAL(NV_CTRL_GVIO_VIDEO_FORMAT_REFRESH_RATE);
-    MAKE_SIGNAL(NV_CTRL_GVO_X_SCREEN_PAN_X);
-    MAKE_SIGNAL(NV_CTRL_GVO_X_SCREEN_PAN_Y);
     MAKE_SIGNAL(NV_CTRL_GPU_OVERCLOCKING_STATE);
     MAKE_SIGNAL(NV_CTRL_GPU_2D_CLOCK_FREQS);
     MAKE_SIGNAL(NV_CTRL_GPU_3D_CLOCK_FREQS);
     MAKE_SIGNAL(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS);
     MAKE_SIGNAL(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE);
+    MAKE_SIGNAL(NV_CTRL_FLATPANEL_LINK);
     MAKE_SIGNAL(NV_CTRL_USE_HOUSE_SYNC);
     MAKE_SIGNAL(NV_CTRL_IMAGE_SETTINGS);
     MAKE_SIGNAL(NV_CTRL_XINERAMA_STEREO);
     MAKE_SIGNAL(NV_CTRL_BUS_RATE);
-    MAKE_SIGNAL(NV_CTRL_SHOW_SLI_HUD);
+    MAKE_SIGNAL(NV_CTRL_SHOW_SLI_VISUAL_INDICATOR);
     MAKE_SIGNAL(NV_CTRL_XV_SYNC_TO_DISPLAY);
     MAKE_SIGNAL(NV_CTRL_GVO_OVERRIDE_HW_CSC);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_TERMINATION);
@@ -232,7 +218,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_MAX_SCREEN_WIDTH);
     MAKE_SIGNAL(NV_CTRL_MAX_SCREEN_HEIGHT);
     MAKE_SIGNAL(NV_CTRL_MAX_DISPLAYS);
-    MAKE_SIGNAL(NV_CTRL_DYNAMIC_TWINVIEW);
     MAKE_SIGNAL(NV_CTRL_MULTIGPU_DISPLAY_OWNER);
     MAKE_SIGNAL(NV_CTRL_GPU_SCALING);
     MAKE_SIGNAL(NV_CTRL_GPU_SCALING_DEFAULT_TARGET);
@@ -313,12 +298,41 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_READING);
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_PROVIDER);
     MAKE_SIGNAL(NV_CTRL_THERMAL_SENSOR_TARGET);
+    MAKE_SIGNAL(NV_CTRL_SHOW_MULTIGPU_VISUAL_INDICATOR);
     MAKE_SIGNAL(NV_CTRL_GPU_CURRENT_PROCESSOR_CLOCK_FREQS);
     MAKE_SIGNAL(NV_CTRL_GVIO_VIDEO_FORMAT_FLAGS);
     MAKE_SIGNAL(NV_CTRL_GPU_PCIE_MAX_LINK_SPEED);
-
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_MODE);
+    MAKE_SIGNAL(NV_CTRL_SYNCHRONOUS_PALETTE_UPDATES);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_TRANSCEIVER_CHANNEL_QUALITY);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_MISSED_SYNC_CYCLES);
+    MAKE_SIGNAL(NV_CTRL_GVO_ANC_PARITY_COMPUTATION);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_PAIR_EVENT);
+    MAKE_SIGNAL(NV_CTRL_3D_VISION_PRO_GLASSES_UNPAIR_EVENT);
+    MAKE_SIGNAL(NV_CTRL_GPU_PCIE_MAX_LINK_WIDTH);
+    MAKE_SIGNAL(NV_CTRL_GPU_PCIE_CURRENT_LINK_WIDTH);
+    MAKE_SIGNAL(NV_CTRL_GPU_PCIE_CURRENT_LINK_SPEED);
+    MAKE_SIGNAL(NV_CTRL_GVO_AUDIO_BLANKING);
+    MAKE_SIGNAL(NV_CTRL_CURRENT_METAMODE_ID);
+    MAKE_SIGNAL(NV_CTRL_DISPLAY_ENABLED);
+    MAKE_SIGNAL(NV_CTRL_FRAMELOCK_INCOMING_HOUSE_SYNC_RATE);
+    MAKE_SIGNAL(NV_CTRL_FXAA);
+    MAKE_SIGNAL(NV_CTRL_DISPLAY_RANDR_OUTPUT_ID);
+    MAKE_SIGNAL(NV_CTRL_FRAMELOCK_DISPLAY_CONFIG);
+    MAKE_SIGNAL(NV_CTRL_TOTAL_DEDICATED_GPU_MEMORY);
+    MAKE_SIGNAL(NV_CTRL_USED_DEDICATED_GPU_MEMORY);
+    MAKE_SIGNAL(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_IMMEDIATE);
+    MAKE_SIGNAL(NV_CTRL_GPU_DOUBLE_PRECISION_BOOST_REBOOT);
+    MAKE_SIGNAL(NV_CTRL_DPY_HDMI_3D);
+    MAKE_SIGNAL(NV_CTRL_BASE_MOSAIC);
+    MAKE_SIGNAL(NV_CTRL_MULTIGPU_MASTER_POSSIBLE);
+    MAKE_SIGNAL(NV_CTRL_GPU_POWER_MIZER_DEFAULT_MODE);
+    MAKE_SIGNAL(NV_CTRL_XV_SYNC_TO_DISPLAY_ID);
+    MAKE_SIGNAL(NV_CTRL_BACKLIGHT_BRIGHTNESS);
+    MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_SPEED);
 #undef MAKE_SIGNAL
-    
+
     /*
      * When new integer attributes are added to NVCtrl.h, a
      * MAKE_SIGNAL() line should be added above.  The below #if should
@@ -326,7 +340,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_CURRENT_DITHERING_DEPTH
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_THERMAL_COOLER_SPEED
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -364,18 +378,29 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_VALID_HORIZ_SYNC_RANGES);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_VALID_VERT_REFRESH_RANGES);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_XINERAMA_SCREEN_INFO);
-    MAKE_STRING_SIGNAL(NV_CTRL_STRING_TWINVIEW_XINERAMA_INFO_ORDER);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_NVIDIA_XINERAMA_INFO_ORDER);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_SLI_MODE);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_PERFORMANCE_MODES);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_GVIO_VIDEO_FORMAT_NAME);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_CURRENT_CLOCK_FREQS);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_3D_VISION_PRO_GLASSES_NAME);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_CURRENT_METAMODE_VERSION_2);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_DP_GUID);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_EDID_HASH);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_TARGET_INDEX);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_RANDR);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_UUID);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_UTILIZATION);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_MULTIGPU_MODE);
 #undef MAKE_STRING_SIGNAL
 
-#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_GPU_CURRENT_CLOCK_FREQS
+#if NV_CTRL_STRING_LAST_ATTRIBUTE != NV_CTRL_STRING_MULTIGPU_MODE
 #warning "There are attributes that do not emit signals!"
 #endif
-    
-    
+
+
     /* make signals for binary attribute */
     for (i = 0; i <= NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE; i++) binary_signals[i] = 0;
 
@@ -397,10 +422,15 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_COOLERS_USED_BY_GPU);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_THERMAL_SENSORS_USED_BY_GPU);
-
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAY_TARGETS);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_METAMODES_VERSION_2);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ENABLED_ON_XSCREEN);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ASSIGNED_TO_XSCREEN);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_GPU_FLAGS);
 #undef MAKE_BINARY_SIGNAL
-    
-#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_THERMAL_SENSORS_USED_BY_GPU
+
+#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_GPU_FLAGS
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -456,7 +486,9 @@ static void ctk_event_register_source(CtkEvent *ctk_event)
             ctk_event_prepare,
             ctk_event_check,
             ctk_event_dispatch,
-            NULL
+            NULL, /* finalize */
+            NULL, /* closure_callback */
+            NULL, /* closure_marshal */
         };
 
         source = g_source_new(&ctk_source_funcs, sizeof(CtkEventSource));
@@ -537,14 +569,14 @@ GtkObject *ctk_event_new(NvCtrlAttributeHandle *handle)
 
 static gboolean ctk_event_prepare(GSource *source, gint *timeout)
 {
+    CtkEventSource *event_source = (CtkEventSource *) source;
     *timeout = -1;
-    
+
     /*
-     * XXX We could check if any events are pending on the Display
-     * connection
+     * Check if any events are pending on the Display connection
      */
-    
-    return FALSE;
+
+    return XPending(event_source->dpy);
 }
 
 
@@ -601,6 +633,8 @@ static gboolean ctk_event_dispatch(GSource *source,
     CtkEventSource *event_source = (CtkEventSource *) source;
     CtkEventStruct event_struct;
 
+    memset(&event_struct, 0, sizeof(event_struct));
+
     /*
      * if ctk_event_dispatch() is called, then either
      * ctk_event_prepare() or ctk_event_check() returned TRUE, so we
@@ -621,14 +655,12 @@ static gboolean ctk_event_dispatch(GSource *source,
 
         /* make sure the attribute is in our signal array */
 
-        if ((nvctrlevent->attribute >= 0) &&
-            (nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
+        if ((nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
             (signals[nvctrlevent->attribute] != 0)) {
             
             event_struct.attribute    = nvctrlevent->attribute;
             event_struct.value        = nvctrlevent->value;
             event_struct.display_mask = nvctrlevent->display_mask;
-            event_struct.availability = TRUE;
 
             /*
              * XXX Is emitting a signal with g_signal_emit() really
@@ -655,15 +687,13 @@ static gboolean ctk_event_dispatch(GSource *source,
 
         /* make sure the attribute is in our signal array */
 
-        if ((nvctrlevent->attribute >= 0) &&
-            (nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
+        if ((nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
             (signals[nvctrlevent->attribute] != 0)) {
             
             event_struct.attribute    = nvctrlevent->attribute;
             event_struct.value        = nvctrlevent->value;
             event_struct.display_mask = nvctrlevent->display_mask;
-            event_struct.availability = TRUE;
-            
+
             /*
              * XXX Is emitting a signal with g_signal_emit() really
              * the "correct" way of dispatching the event?
@@ -690,15 +720,14 @@ static gboolean ctk_event_dispatch(GSource *source,
 
         /* make sure the attribute is in our signal array */
 
-        if ((nvctrlevent->attribute >= 0) &&
-            (nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
+        if ((nvctrlevent->attribute <= NV_CTRL_LAST_ATTRIBUTE) &&
             (signals[nvctrlevent->attribute] != 0)) {
             
             event_struct.attribute    = nvctrlevent->attribute;
             event_struct.value        = nvctrlevent->value;
             event_struct.display_mask = nvctrlevent->display_mask;
-            event_struct.availability = nvctrlevent->availability;
-            
+            event_struct.is_availability_changed = TRUE;
+
             /*
              * XXX Is emitting a signal with g_signal_emit() really
              * the "correct" way of dispatching the event?
@@ -722,14 +751,12 @@ static gboolean ctk_event_dispatch(GSource *source,
 
         /* make sure the attribute is in our signal array */
         
-        if ((nvctrlevent->attribute >= 0) &&
-            (nvctrlevent->attribute <= NV_CTRL_STRING_LAST_ATTRIBUTE) &&
+        if ((nvctrlevent->attribute <= NV_CTRL_STRING_LAST_ATTRIBUTE) &&
             (string_signals[nvctrlevent->attribute] != 0)) {
 
             event_struct.attribute    = nvctrlevent->attribute;
             event_struct.value        = 0;
             event_struct.display_mask = nvctrlevent->display_mask;
-            event_struct.availability = TRUE;
             /*
              * XXX Is emitting a signal with g_signal_emit() really
              * the "correct" way of dispatching the event
@@ -752,14 +779,12 @@ static gboolean ctk_event_dispatch(GSource *source,
             (XNVCtrlBinaryAttributeChangedEventTarget *) &event;
 
         /* make sure the attribute is in our signal array */
-        if ((nvctrlevent->attribute >= 0) &&
-            (nvctrlevent->attribute <= NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE) &&
+        if ((nvctrlevent->attribute <= NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE) &&
             (binary_signals[nvctrlevent->attribute] != 0)) {
 
             event_struct.attribute    = nvctrlevent->attribute;
             event_struct.value        = 0;
             event_struct.display_mask = nvctrlevent->display_mask;
-            event_struct.availability = TRUE;
             /*
              * XXX Is emitting a signal with g_signal_emit() really
              * the "correct" way of dispatching the event
@@ -811,7 +836,7 @@ static gboolean ctk_event_dispatch(GSource *source,
 
 
 /* ctk_event_emit() - Emits signal(s) on a registered ctk_event object.
- * This function is primarly used to simulate NV-CONTROL events such
+ * This function is primarily used to simulate NV-CONTROL events such
  * that various parts of nvidia-settings can communicate (internally)
  */
 void ctk_event_emit(CtkEvent *ctk_event,
@@ -836,7 +861,7 @@ void ctk_event_emit(CtkEvent *ctk_event,
     if (!source) return;
 
 
-    /* Broadcast event to all relavent ctk_event objects */
+    /* Broadcast event to all relevant ctk_event objects */
     event.attribute = attrib;
     event.value = value;
     event.display_mask = mask;
@@ -852,7 +877,7 @@ void ctk_event_emit(CtkEvent *ctk_event,
 
 
 /* ctk_event_emit_string() - Emits signal(s) on a registered ctk_event object.
- * This function is primarly used to simulate NV-CONTROL events such
+ * This function is primarily used to simulate NV-CONTROL events such
  * that various parts of nvidia-settings can communicate (internally)
  */
 void ctk_event_emit_string(CtkEvent *ctk_event,
@@ -877,7 +902,7 @@ void ctk_event_emit_string(CtkEvent *ctk_event,
     if (!source) return;
 
 
-    /* Broadcast event to all relavent ctk_event objects */
+    /* Broadcast event to all relevant ctk_event objects */
 
     event.attribute = attrib;
     event.value = 0;
