@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <stdlib.h>
@@ -114,7 +109,7 @@ static const char * __cav_help =
   "cav (Caveats) - Caveats for this configuration.  A frame buffer "
   "configuration may have the following caveats: 'NonC' if it supports "
   "any non-conformant visual extension.  'Slow' if it has reduced "
-  "preformance.  '-' if it has no caveats.";
+  "performance.  '-' if it has no caveats.";
 static const char * __pbw_help =
   "pbw (Pbuffer width) - Width of pbuffer (in hexadecimal).";
 static const char * __pbh_help =
@@ -125,15 +120,15 @@ static const char * __pbp_help =
 static const char * __trt_help =
   "trt (Transparency type) - Type of transparency (RGBA or Index).";
 static const char * __trr_help =
-  "trr (Transparency red value) - Red value consided transparent.";
+  "trr (Transparency red value) - Red value considered transparent.";
 static const char * __trg_help =
-  "trg (Transparency green value) - Green value consided transparent.";
+  "trg (Transparency green value) - Green value considered transparent.";
 static const char * __trb_help =
-  "trb (Transparency blue value) - Blue value consided transparent.";
+  "trb (Transparency blue value) - Blue value considered transparent.";
 static const char * __tra_help =
-  "tra (Transparency alpha value) - Alpha value consided transparent.";
+  "tra (Transparency alpha value) - Alpha value considered transparent.";
 static const char * __tri_help =
-  "tri (Transparency index value) - Color index value consided transparent.";
+  "tri (Transparency index value) - Color index value considered transparent.";
 
 
 GType ctk_glx_get_type(void)
@@ -151,6 +146,7 @@ GType ctk_glx_get_type(void)
             sizeof (CtkGLX),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_glx_type = g_type_register_static (GTK_TYPE_VBOX,
@@ -237,7 +233,7 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     GtkWidget *banner;
     GtkWidget *hseparator;
     GtkWidget *hbox;
-    GtkWidget *vbox, *vbox2;
+    GtkWidget *vbox;
     GtkWidget *scrollWin;
     GtkWidget *event;    /* For setting the background color to white */
     GtkWidget *data_table, *header_table;
@@ -314,10 +310,9 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
     hbox = gtk_hbox_new(FALSE, 0);
     vbox = gtk_vbox_new(FALSE, 5);
     event = gtk_event_box_new();
+    ctk_force_text_colors_on_widget(event);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin),
                                    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, &(event->style->text[GTK_STATE_NORMAL]));
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &(event->style->base[GTK_STATE_NORMAL]));
     gtk_container_add(GTK_CONTAINER(event), hbox);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin),
                                           event);
@@ -373,7 +368,6 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 
     hbox      = gtk_hbox_new(FALSE, 0);
     vbox      = gtk_vbox_new(FALSE, 10);
-    vbox2     = gtk_vbox_new(FALSE, 10);
 
     data_viewport = gtk_viewport_new(NULL, NULL);
     gtk_widget_set_size_request(data_viewport, 400, 50);
@@ -412,16 +406,8 @@ GtkWidget* ctk_glx_new(NvCtrlAttributeHandle *handle,
 
     data_table = gtk_table_new(num_fbconfigs, NUM_FBCONFIG_ATTRIBS, FALSE);
     event = gtk_event_box_new();
-    
-    gtk_widget_modify_fg(data_table, GTK_STATE_NORMAL, 
-                         &(data_table->style->text[GTK_STATE_NORMAL]));
-    gtk_widget_modify_bg(data_table, GTK_STATE_NORMAL, 
-                         &(data_table->style->base[GTK_STATE_NORMAL]));
+    ctk_force_text_colors_on_widget(event);
     gtk_container_add (GTK_CONTAINER(event), data_table);
-    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, 
-                         &(event->style->text[GTK_STATE_NORMAL]));
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, 
-                         &(event->style->base[GTK_STATE_NORMAL]));
     gtk_container_add(GTK_CONTAINER(data_viewport), event);
 
     /* Pack the fbconfig header and data tables */
@@ -950,7 +936,7 @@ GtkTextBuffer *ctk_glx_create_help(GtkTextTagTable *table,
                  );
 
     ctk_help_heading(b, &i, "Show GLX Frame Buffer Configurations");
-    ctk_help_para(b, &i, __show_fbc_help);
+    ctk_help_para(b, &i, "%s", __show_fbc_help);
 
     ctk_help_heading(b, &i, "GLX Frame Buffer Configurations");
     ctk_help_para(b, &i, "This table lists the supported frame buffer "
