@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2009 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <stdlib.h>
@@ -36,6 +31,7 @@
 #include "ctkgvi.h"
 #include "ctkgpu.h"
 #include "ctkbanner.h"
+#include "ctkdropdownmenu.h"
 
 #define DEFAULT_UPDATE_VIDEO_FORMAT_INFO_TIME_INTERVAL 1000
 
@@ -56,6 +52,7 @@ GType ctk_gvi_get_type(void)
             sizeof (CtkGvi),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_gvi_type =
@@ -101,8 +98,8 @@ static gboolean update_sdi_input_info(gpointer user_data);
 /*
  * ctk_gvio_get_format_name() - retrun name of format.
  */
-const char * ctk_gvio_get_format_name(const GvioFormatName *formatTable,
-                                      const gint format)
+static const char *ctk_gvio_get_format_name(const GvioFormatName *formatTable,
+                                            const gint format)
 {
     int i;
     for (i = 0; formatTable[i].name; i++) {
@@ -264,20 +261,17 @@ static void update_sdi_input_info_simple(CtkGvi *ctk_gvi)
 }
 
 
-static void jack_channel_changed(GtkOptionMenu *optionmenu,
+static void jack_channel_changed(GtkWidget *widget,
                                  gpointer user_data)
 {
     CtkGvi *ctk_gvi = CTK_GVI(user_data);
-    GtkWidget *menu;
-    GtkWidget *menu_item;
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
+    gint idx;
 
     /* Track new selection */
-    menu = gtk_option_menu_get_menu(optionmenu);
-    menu_item = gtk_menu_get_active(GTK_MENU(menu));
-
-    ctk_gvi->cur_jack_channel =
-        GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(menu_item),
-                                           "JACK_CHANNEL"));
+    idx = ctk_drop_down_menu_get_current_value(menu);
+    
+    ctk_gvi->cur_jack_channel = ctk_gvi->jack_channel_table[idx];
 
     update_sdi_input_info(ctk_gvi);
 }
@@ -285,9 +279,7 @@ static void jack_channel_changed(GtkOptionMenu *optionmenu,
 
 static GtkWidget *create_jack_channel_menu(CtkGvi *ctk_gvi)
 {
-    GtkWidget *omenu;
-    GtkWidget *menu;
-    GtkWidget *menu_item;
+    CtkDropDownMenu *menu;
     gint idx;
     gchar *label_str;
     gint jack;
@@ -297,9 +289,8 @@ static GtkWidget *create_jack_channel_menu(CtkGvi *ctk_gvi)
 
     /* Create the menu */
 
-    omenu = gtk_option_menu_new();
-
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
     
     /* Just show all jack/channel pairs in dropdown */
 
@@ -314,16 +305,11 @@ static GtkWidget *create_jack_channel_menu(CtkGvi *ctk_gvi)
 
             label_str = g_strdup_printf("Jack %d, Channel %d",
                                         jack+1, channel+1);
-            menu_item = gtk_menu_item_new_with_label(label_str);
+            ctk_drop_down_menu_append_item(menu, label_str, idx);
             g_free(label_str);
 
-            g_object_set_data(G_OBJECT(menu_item),
-                              "JACK_CHANNEL",
-                              GUINT_TO_POINTER(jack_channel));
-
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-            gtk_widget_show(menu_item);
-
+            ctk_gvi->jack_channel_table[idx] = jack_channel;
+            
             if (jack_channel == ctk_gvi->cur_jack_channel) {
                 selected_idx = idx;
             }
@@ -332,15 +318,13 @@ static GtkWidget *create_jack_channel_menu(CtkGvi *ctk_gvi)
         }
     }
 
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+    ctk_drop_down_menu_set_current_value(menu, selected_idx);
 
-    gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), selected_idx);
-
-    g_signal_connect(G_OBJECT(omenu), "changed",
+    g_signal_connect(G_OBJECT(menu), "changed",
                      G_CALLBACK(jack_channel_changed),
                      (gpointer) ctk_gvi);
 
-    return omenu;
+    return GTK_WIDGET(menu);
 }
 
 
@@ -554,7 +538,7 @@ GtkWidget* ctk_gvi_new(NvCtrlAttributeHandle *handle,
     /* Get Bus related information */
 
     get_bus_type_str(handle, &bus);
-    get_bus_id_str(handle, &pci_bus_id);
+    pci_bus_id = get_bus_id_str(handle);
 
     /* NV_CTRL_IRQ */
 
@@ -666,6 +650,14 @@ GtkWidget* ctk_gvi_new(NvCtrlAttributeHandle *handle,
 
     hsep = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(hbox), hsep, TRUE, TRUE, 5);
+    
+    /* Create look up table to store jack chanel */
+    if (ctk_gvi->jack_channel_table) {
+        free(ctk_gvi->jack_channel_table);
+    }
+    ctk_gvi->jack_channel_table =
+                calloc(ctk_gvi->max_channels_per_jack * ctk_gvi->num_jacks,
+                       sizeof(unsigned int));
 
     /* Jack+Channel selection dropdown (hidden in condensed view) */
     
@@ -752,13 +744,21 @@ GtkTextBuffer *ctk_gvi_create_help(GtkTextTagTable *table,
     ctk_help_para(b, &i, "This is the interrupt request line assigned to "
                   "this GVI device.");
 
+    ctk_help_heading(b, &i, "Bound GPU");
+    ctk_help_para(b, &i, "An OpenGL application can bind a GVI device to a "
+                  "GPU using the GL_NV_video_capture OpenGL extension.  The "
+                  "Bound GPU field reports if an OpenGL application has "
+                  "currently bound this GVI device to a GPU.");
+
     ctk_help_heading(b, &i, "Input Information");
     ctk_help_para(b, &i, "This section shows the detected video format(s) on "
                   "each jack of the GVI device.  When condensed mode is "
                   "selected, the detected video format is shown for each "
-                  "jack (and channel).  when detailed mode is selected, "
-                  "information pertaining to the single channel on a jack "
-                  "selected is given as such:");
+                  "jack (and channel).  When detailed mode is selected, "
+                  "information pertaining to the selected jack is reported.  "
+                  "Note that the GVI device can only detect the following "
+                  "information if the incoming signal has a non-zero SMPTE "
+                  "352 payload identifier, which not all SDI devices provide.");
 
     ctk_help_para(b, &i, "Video Format:  The detected SMPTE video format.");
     ctk_help_para(b, &i, "Component Sampling: The detected composition of the "
