@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -170,6 +165,7 @@ GType ctk_gvo_sync_get_type(void)
             sizeof (CtkGvoSync),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_gvo_sync_type =
@@ -680,7 +676,7 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
 
 
 /*
- * start_menu() - Start the creation of a labled dropdown menu.  (Packs
+ * start_menu() - Start the creation of a labeled dropdown menu.  (Packs
  * the dropdown label into the table row.
  */
 
@@ -906,7 +902,7 @@ static void register_for_gvo_sync_events(CtkGvoSync *ctk_gvo_sync)
 
 /*
  * update_gvo_sync_sensitivity() - Updates the sensitivity of various UI
- * wigets based on whether or not an input sync signal is detected.
+ * widgets based on whether or not an input sync signal is detected.
  *
  */
 
@@ -1011,7 +1007,7 @@ static void post_sync_mode_menu_changed(CtkGvoSync *ctk_gvo_sync, gint value)
 
 
 /*
- * post_sync_format_menu_changed() - Call this function after teh sync
+ * post_sync_format_menu_changed() - Call this function after the sync
  * format has changed.
  */
 
@@ -1183,17 +1179,9 @@ static void sync_mode_changed(CtkDropDownMenu *menu, gpointer user_data)
 {
     CtkGvoSync *ctk_gvo_sync = CTK_GVO_SYNC(user_data);
     gint value;
-    char *name;
-    
+
     value = ctk_drop_down_menu_get_current_value(menu);
-    
-    switch (value) {
-    case NV_CTRL_GVO_SYNC_MODE_FREE_RUNNING: name = "Free Running"; break;
-    case NV_CTRL_GVO_SYNC_MODE_GENLOCK:      name = "GenLock";      break;
-    case NV_CTRL_GVO_SYNC_MODE_FRAMELOCK:    name = "FrameLock";    break;
-    default: return;
-    }
-    
+
     NvCtrlSetAttribute(ctk_gvo_sync->handle, NV_CTRL_GVO_SYNC_MODE, value);
     
     if (value != NV_CTRL_GVO_SYNC_MODE_FREE_RUNNING) {
@@ -1514,30 +1502,30 @@ GtkTextBuffer* ctk_gvo_sync_create_help(GtkTextTagTable *table,
                   "SDI synchronization options.");
 
     ctk_help_heading(b, &i, "Input Video Format");
-    ctk_help_para(b, &i, __input_video_format_help);
+    ctk_help_para(b, &i, "%s", __input_video_format_help);
     ctk_help_heading(b, &i, "Input Video Format Detect");
-    ctk_help_para(b, &i, __input_video_format_detect_help);
+    ctk_help_para(b, &i, "%s", __input_video_format_detect_help);
     ctk_help_heading(b, &i, "Composite Termination");
     ctk_help_para(b, &i, "%s.  This allows the composite signal to be daisy "
                   "chained from a server load.",
                   __composite_termination_help);
     ctk_help_heading(b, &i, "Sync Mode");
-    ctk_help_para(b, &i, __sync_mode_help);
+    ctk_help_para(b, &i, "%s", __sync_mode_help);
     ctk_help_heading(b, &i, "Sync Format");
-    ctk_help_para(b, &i, __sync_format_help);
+    ctk_help_para(b, &i, "%s", __sync_format_help);
     ctk_help_heading(b, &i, "Sync Status");
-    ctk_help_para(b, &i, __sync_status_help);
+    ctk_help_para(b, &i, "%s", __sync_status_help);
 
     if ( ctk_gvo_sync->caps & NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
         ctk_help_heading(b, &i, "HSync Advance");
-        ctk_help_para(b, &i, __hsync_advance_help);
+        ctk_help_para(b, &i, "%s", __hsync_advance_help);
         ctk_help_heading(b, &i, "VSync Advance");
-        ctk_help_para(b, &i, __vsync_advance_help);
+        ctk_help_para(b, &i, "%s", __vsync_advance_help);
     } else {
         ctk_help_heading(b, &i, "HSync Delay");
-        ctk_help_para(b, &i, __hsync_delay_help);
+        ctk_help_para(b, &i, "%s", __hsync_delay_help);
         ctk_help_heading(b, &i, "VSync Delay");
-        ctk_help_para(b, &i, __vsync_delay_help);
+        ctk_help_para(b, &i, "%s", __vsync_delay_help);
     }
 
     ctk_help_finish(b);
