@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -51,7 +46,8 @@ static const _CtkStereoMode stereoMode[] = {
     { NV_CTRL_STEREO_CHECKERBOARD_PATTERN,         "Checkerboard Pattern Stereo" },
     { NV_CTRL_STEREO_INVERSE_CHECKERBOARD_PATTERN, "Inverse Checkerboard Stereo" },
     { NV_CTRL_STEREO_3D_VISION,                    "NVIDIA 3D Vision Stereo" },
-    { -1, NULL},
+    { NV_CTRL_STEREO_3D_VISION_PRO,                "NVIDIA 3D Vision Pro Stereo" },
+    { NV_CTRL_STEREO_HDMI_3D,                      "HDMI 3D Stereo" },
 };
 
 void ctk_screen_event_handler(GtkWidget *widget,
@@ -67,7 +63,7 @@ static void info_update_gpu_error(GtkObject *object, gpointer arg1,
 static const char *get_stereo_mode_string(int stereo_mode)
 {
     int i;
-    for (i = 0; stereoMode[i].name; i++) {
+    for (i = 0; i < ARRAY_LEN(stereoMode); i++) {
         if (stereoMode[i].stereo_mode == stereo_mode) {
             return stereoMode[i].name;
         }
@@ -92,6 +88,7 @@ GType ctk_screen_get_type(
             sizeof (CtkScreen),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_screen_type =
@@ -104,156 +101,26 @@ GType ctk_screen_get_type(
 
 
 
-/* Generates a list of display device names for those display devices that
- * are enabled on the given GPU and, if Xinerama is disabled, are also
- * associated to the given X screen (handle).
- */
-static gchar *make_gpu_display_device_list(NvCtrlAttributeHandle *handle,
-                                           int gpu_id,
-                                           int xinerama_enabled)
-{
-    gchar *displays = NULL;
-    gchar *type;
-    gchar *name;
-    gchar *tmp_str;
-    unsigned int display_devices;
-    unsigned int mask;
-    Bool valid;
-
-
-    /* Get the list of enabled display devices on the GPU */
-
-    valid =
-        XNVCTRLQueryTargetAttribute(NvCtrlGetDisplayPtr(handle),
-                                    NV_CTRL_TARGET_TYPE_GPU,
-                                    gpu_id,
-                                    0,
-                                    NV_CTRL_ENABLED_DISPLAYS,
-                                    (int *)&display_devices);
-    if (!valid) return NULL;
-
-    /* If Xinerama is disabled, only show displays that are associated
-     * to this X screen.
-     */
-    if (!xinerama_enabled) {
-        ReturnStatus ret;
-        unsigned int associated_devices;
-
-        ret = NvCtrlGetAttribute(handle,
-                                 NV_CTRL_ASSOCIATED_DISPLAY_DEVICES,
-                                 (int *)&associated_devices);
-        if (ret == NvCtrlSuccess) {
-            display_devices &= associated_devices;
-        }
-    }
-
-    /* Make the list of display device names */
-
-    for (mask = 1; mask; mask <<= 1) {
-
-        if (!(mask & display_devices)) continue;
-
-        type = display_device_mask_to_display_device_name(mask);
-        name = NULL;
-
-        valid =
-            XNVCTRLQueryTargetStringAttribute(NvCtrlGetDisplayPtr(handle),
-                                              NV_CTRL_TARGET_TYPE_GPU,
-                                              gpu_id,
-                                              mask,
-                                              NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
-                                              &name);
-        if (!valid) {
-            tmp_str = g_strdup_printf("Unknown (%s)", type);
-        } else {
-            tmp_str = g_strdup_printf("%s (%s)", name, type);
-            XFree(name);
-        }
-        free(type);
-
-        if (displays) {
-            name = g_strdup_printf("%s,\n%s", tmp_str, displays);
-            g_free(displays);
-            g_free(tmp_str);
-        } else {
-            name = tmp_str;
-        }
-        displays = name;
-    }
-
-    return displays;
-}
-
-
-
 /* Generates a list of display devices for the logical X screen
  * given as "handle".
  */
 static gchar *make_display_device_list(NvCtrlAttributeHandle *handle)
 {
-    ReturnStatus ret;
-    int len;
-    int i;
-    int *pData;
-    gchar *displays = NULL;
-    int xinerama_enabled;
-
-
-    /* See if Xinerama is enabled */
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_XINERAMA, &xinerama_enabled);
-    if (ret != NvCtrlSuccess) {
-        goto done;
-    }
-
-    /* Get all GPUs driving this X screen */
-    ret = NvCtrlGetBinaryAttribute(handle,
-                                   0,
-                                   NV_CTRL_BINARY_DATA_GPUS_USED_BY_LOGICAL_XSCREEN,
-                                   (unsigned char **)(&pData),
-                                   &len);
-    if (ret != NvCtrlSuccess) {
-        goto done;
-    }
-
-    /* Generate the list of display device names that display this X screen */
-    for (i = 1; i <= pData[0]; i++) {
-        gchar *new_str;
-        gchar *tmp_str;
-
-        new_str = make_gpu_display_device_list(handle, pData[i],
-                                               xinerama_enabled);
-        if (new_str) {
-            if (displays) {
-                tmp_str = g_strdup_printf("%s,\n%s", displays, new_str);
-                g_free(displays);
-                g_free(new_str);
-                displays = tmp_str;
-            } else {
-                displays = new_str;
-            }
-        }
-    }
-
- done:
-    if (!displays) {
-        displays = g_strdup("None");
-    }
-
-    return displays;
-
+    return create_display_name_list_string(handle,
+                                           NV_CTRL_BINARY_DATA_DISPLAYS_ENABLED_ON_XSCREEN);
 } /* make_display_device_list() */
 
 
 
 /*
  * Calculations of the screen dimensions and resolution are based on
- * the dxpyinfo utility code.
+ * the xdpyinfo utility code.
  *
  * Copyright Information for xdpyinfo:
  *
  ***********************************************************************
  * 
- * xdpyinfo - print information about X display connecton
+ * xdpyinfo - print information about X display connection
  *
  * 
 Copyright 1988, 1998  The Open Group
@@ -399,20 +266,12 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
     /* get the number of gpu errors occurred */
 
     gpu_errors = 0;
-    ret = NvCtrlGetDisplayAttribute(handle,
-                                    0,
-                                    NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
-                                    (int *)&gpu_errors);
+    ret = NvCtrlGetAttribute(handle,
+                             NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
+                             (int *)&gpu_errors);
 
     snprintf(tmp, 16, "%d", gpu_errors);
 
-    /* get the stereo mode set for this X screen */
-    ret = NvCtrlGetDisplayAttribute(handle, 0, NV_CTRL_STEREO,
-                                    (int *)&stereo_mode);
-    if (ret != NvCtrlSuccess) {
-        stereo_mode = -1;
-    }
-
     /* now, create the object */
     
     object = g_object_new(CTK_TYPE_SCREEN, NULL);
@@ -422,6 +281,10 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
 
     ctk_screen->handle = handle;
 
+    /* get the stereo mode set for this X screen */
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_STEREO, (int *)&stereo_mode);
+    ctk_screen->stereo_available = (ret == NvCtrlSuccess);
+
     /* set container properties of the object */
 
     gtk_box_set_spacing(GTK_BOX(ctk_screen), 10);
@@ -474,8 +337,10 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
     /* gpu errors */
     ctk_screen->gpu_errors =
         add_table_row(table, 19, 0, 0, "Recovered GPU Errors:", 0, 0, tmp);
-    add_table_row(table, 20, 0, 0, "Stereo Mode:", 0, 0,
-                  get_stereo_mode_string(stereo_mode));
+    if (ctk_screen->stereo_available) {
+        add_table_row(table, 20, 0, 0, "Stereo Mode:", 0, 0,
+                      get_stereo_mode_string(stereo_mode));
+    }
 
     g_free(screen_number);
     free(display_name);
@@ -510,6 +375,7 @@ GtkWidget* ctk_screen_new(NvCtrlAttributeHandle *handle,
 
     
 GtkTextBuffer *ctk_screen_create_help(GtkTextTagTable *table,
+                                      CtkScreen *ctk_screen,
                                       const gchar *screen_name)
 {
     GtkTextIter i;
@@ -562,8 +428,10 @@ GtkTextBuffer *ctk_screen_create_help(GtkTextTagTable *table,
                   "GPU received and the NVIDIA X driver successfully recovered "
                   "from.");
 
-    ctk_help_heading(b, &i, "Stereo Mode");
-    ctk_help_para(b, &i, "This is the stereo mode set for the X screen.");
+    if (ctk_screen->stereo_available) {
+        ctk_help_heading(b, &i, "Stereo Mode");
+        ctk_help_para(b, &i, "This is the stereo mode set for the X screen.");
+    }
 
     ctk_help_finish(b);
 
@@ -616,7 +484,7 @@ static void associated_displays_received(GtkObject *object, gpointer arg1,
 
 
 /*
- * When the  number of gpu errors occured changes,
+ * When the number of gpu errors occurred changes,
  * update the count showed on the page.
  */
 
@@ -629,11 +497,12 @@ static void info_update_gpu_error(GtkObject *object, gpointer arg1,
     char tmp[16];
 
     /* get the number of gpu errors occurred */
-    ret = NvCtrlGetDisplayAttribute(ctk_screen->handle, 0,
-                                    NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
-                                    (int *)&gpu_errors);
+    ret = NvCtrlGetAttribute(ctk_screen->handle,
+                             NV_CTRL_NUM_GPU_ERRORS_RECOVERED,
+                             (int *)&gpu_errors);
     if (ret == NvCtrlSuccess) {
         snprintf(tmp, 16, "%d", gpu_errors);
         gtk_label_set_text(GTK_LABEL(ctk_screen->gpu_errors), tmp);
     }
+
 } /* info_update_gpu_error() */
