@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <gtk/gtk.h>
@@ -102,8 +97,11 @@ static const char * __enable_button_help =
 static const char * __fan_id_help =
 "This shows the GPU Fan's index.";
 
+static const char * __fan_rpm_help =
+"This shows the current GPU Fan Speed in rotations per minute (RPM).";
+
 static const char * __fan_speed_help =
-"This shows the current GPU Fan Speed.";
+"This shows the current GPU Fan Speed level as a percentage.";
 
 static const char * __fan_control_type_help =
 "Fan Type indicates if and how this fan may be controlled.  Possible "
@@ -141,6 +139,7 @@ GType ctk_thermal_get_type(void)
             sizeof (CtkThermal),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_thermal_type =
@@ -159,7 +158,7 @@ GType ctk_thermal_get_type(void)
  */
 static gboolean update_cooler_info(gpointer user_data)
 {
-    int i, level, cooler_type, cooler_target;
+    int i, speed, level, cooler_type, cooler_target;
     gchar *tmp_str;
     CtkThermal *ctk_thermal;
     GtkWidget *table, *label, *eventbox;
@@ -194,18 +193,26 @@ static gboolean update_cooler_info(gpointer user_data)
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_id_help);
 
-    label = gtk_label_new("Speed (%)");
+    label = gtk_label_new("Speed (RPM)");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
     gtk_table_attach(GTK_TABLE(table), eventbox, 1, 2, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
+    ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_rpm_help);
+
+    label = gtk_label_new("Speed (%)");
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+    eventbox = gtk_event_box_new();
+    gtk_table_attach(GTK_TABLE(table), eventbox, 2, 3, 0, 1,
+                     GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
+    gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox, __fan_speed_help);
 
     label = gtk_label_new("Control Type");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
-    gtk_table_attach(GTK_TABLE(table), eventbox, 2, 3, 0, 1,
+    gtk_table_attach(GTK_TABLE(table), eventbox, 3, 4, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox,
@@ -214,7 +221,7 @@ static gboolean update_cooler_info(gpointer user_data)
     label = gtk_label_new("Cooling Target");
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     eventbox = gtk_event_box_new();
-    gtk_table_attach(GTK_TABLE(table), eventbox, 3, 4, 0, 1,
+    gtk_table_attach(GTK_TABLE(table), eventbox, 4, 5, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     ctk_config_set_tooltip(ctk_thermal->ctk_config, eventbox,
@@ -224,7 +231,7 @@ static gboolean update_cooler_info(gpointer user_data)
     for (i = 0; i < ctk_thermal->cooler_count; i++) {
         row_idx = i+1;
 
-        gtk_table_resize(GTK_TABLE(table), row_idx+1, 4);
+        gtk_table_resize(GTK_TABLE(table), row_idx+1, 5);
 
         tmp_str = g_strdup_printf("%d", i);
         label = gtk_label_new(tmp_str);
@@ -233,6 +240,21 @@ static gboolean update_cooler_info(gpointer user_data)
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
+        ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
+                                 NV_CTRL_THERMAL_COOLER_SPEED,
+                                 &speed);
+        if (ret == NvCtrlSuccess) {
+            tmp_str = g_strdup_printf("%d", speed);
+        }
+        else {
+            tmp_str = g_strdup_printf("Unsupported");
+        }
+        label = gtk_label_new(tmp_str);
+        gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
+        gtk_table_attach(GTK_TABLE(table), label, 1, 2, row_idx, row_idx+1,
+                         GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
+        free(tmp_str);
+
         ret = NvCtrlGetAttribute(ctk_thermal->cooler_control[i].handle,
                                  NV_CTRL_THERMAL_COOLER_LEVEL,
                                  &level);
@@ -243,7 +265,7 @@ static gboolean update_cooler_info(gpointer user_data)
         tmp_str = g_strdup_printf("%d", level);
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 1, 2, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 2, 3, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
@@ -263,7 +285,7 @@ static gboolean update_cooler_info(gpointer user_data)
         }
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 2, 3, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 3, 4, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
 
@@ -292,7 +314,7 @@ static gboolean update_cooler_info(gpointer user_data)
         }
         label = gtk_label_new(tmp_str);
         gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
-        gtk_table_attach(GTK_TABLE(table), label, 3, 4, row_idx, row_idx+1,
+        gtk_table_attach(GTK_TABLE(table), label, 4, 5, row_idx, row_idx+1,
                          GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
         free(tmp_str);
     }
@@ -394,7 +416,7 @@ static void cooler_control_state_update_gui(CtkThermal *ctk_thermal)
 
 
     /* We need to check the cooler control state status with 
-     * the server everytime someone tries to change the state
+     * the server every time someone tries to change the state
      * because the set might have failed.
      */
 
@@ -433,7 +455,6 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
 {
     CtkThermal *ctk_thermal = CTK_THERMAL(user_data);
     gboolean enabled;
-    ReturnStatus ret;
     int value;
     gint result;
 
@@ -477,8 +498,8 @@ static void cooler_control_state_toggled(GtkWidget *widget, gpointer user_data)
 
     /* Update the server */
 
-    ret = NvCtrlSetAttribute(ctk_thermal->attribute_handle,
-                             NV_CTRL_GPU_COOLER_MANUAL_CONTROL, value);
+    NvCtrlSetAttribute(ctk_thermal->attribute_handle,
+                       NV_CTRL_GPU_COOLER_MANUAL_CONTROL, value);
 
     /* Update the GUI */
 
@@ -1187,9 +1208,8 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
         ctk_thermal->show_fan_control_frame = FALSE;
         value = NV_CTRL_GPU_COOLER_MANUAL_CONTROL_FALSE;
     }
-    
-    cooler_control_enabled =
-        (value==NV_CTRL_GPU_COOLER_MANUAL_CONTROL_TRUE)?TRUE:FALSE;
+
+    cooler_control_enabled = (value == NV_CTRL_GPU_COOLER_MANUAL_CONTROL_TRUE);
 
     can_access_cooler_level = TRUE;
     ctk_thermal->cooler_control_enabled = cooler_control_enabled;
@@ -1275,7 +1295,7 @@ GtkWidget* ctk_thermal_new(NvCtrlAttributeHandle *handle,
             cur_sensor_idx++;
         }
     } else {
-        /* GPU Core Treshold Temperature */
+        /* GPU Core Threshold Temperature */
 
         vbox1 = gtk_vbox_new(FALSE, 0);
         hbox1 = gtk_hbox_new(FALSE, 0);
@@ -1395,7 +1415,7 @@ sensor_end:
     /* Create cooler level control sliders/checkbox */
     
     ctk_thermal->cooler_control = (CoolerControlPtr)
-        malloc(ctk_thermal->cooler_count * sizeof(CoolerControlRec));
+        calloc(ctk_thermal->cooler_count, sizeof(CoolerControlRec));
 
     for (j = 1; j <= ctk_thermal->cooler_count; j++) {
         cooler_handle = h->targets[COOLER_TARGET].t[pDataCooler[j]].h;
@@ -1477,8 +1497,8 @@ sensor_end:
             gtk_widget_set_sensitive(ctk_thermal->cooler_control
                                      [cur_cooler_idx].widget,
                                      cooler_control_enabled);
-            cur_cooler_idx++;
         }
+        cur_cooler_idx++;
     }
     
     if ( ctk_thermal->cooler_count && ctk_thermal->show_fan_control_frame ) {
@@ -1629,32 +1649,32 @@ GtkTextBuffer *ctk_thermal_create_help(GtkTextTagTable *table,
 
     if (!ctk_thermal->thermal_sensor_target_type_supported) {
         ctk_help_heading(b, &i, "Slowdown Threshold");
-        ctk_help_para(b, &i, __core_threshold_help);
+        ctk_help_para(b, &i, "%s", __core_threshold_help);
 
         ctk_help_heading(b, &i, "Core Temperature");
-        ctk_help_para(b, &i, __core_temp_help);
+        ctk_help_para(b, &i, "%s", __core_temp_help);
 
         if (ctk_thermal->ambient_label) {
             ctk_help_heading(b, &i, "Ambient Temperature");
-            ctk_help_para(b, &i, __ambient_temp_help);
+            ctk_help_para(b, &i, "%s", __ambient_temp_help);
         }
     } else {
         ctk_help_title(b, &i, "Thermal Sensor Information Help");
 
         ctk_help_heading(b, &i, "ID");
-        ctk_help_para(b, &i, __thermal_sensor_id_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_id_help);
 
         ctk_help_heading(b, &i, "Temperature");
-        ctk_help_para(b, &i, __thermal_sensor_reading_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_reading_help);
 
         ctk_help_heading(b, &i, "Target");
-        ctk_help_para(b, &i, __thermal_sensor_target_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_target_help);
         
         ctk_help_heading(b, &i, "Provider");
-        ctk_help_para(b, &i, __thermal_sensor_provider_help);
+        ctk_help_para(b, &i, "%s", __thermal_sensor_provider_help);
     }
     ctk_help_heading(b, &i, "Level");
-    ctk_help_para(b, &i, __temp_level_help);
+    ctk_help_para(b, &i, "%s", __temp_level_help);
 
 next_help:
     /* if Fan not available skip online help */
@@ -1665,26 +1685,29 @@ next_help:
     ctk_help_title(b, &i, "GPU Fan Settings Help");
 
     ctk_help_heading(b, &i, "ID");
-    ctk_help_para(b, &i, __fan_id_help);
+    ctk_help_para(b, &i, "%s", __fan_id_help);
 
-    ctk_help_heading(b, &i, "Speed (%)");
-    ctk_help_para(b, &i, __fan_speed_help);
+    ctk_help_heading(b, &i, "Speed (RPM)");
+    ctk_help_para(b, &i,"%s", __fan_rpm_help);
+
+    ctk_help_heading(b, &i, "Speed (%%)");
+    ctk_help_para(b, &i, "%s", __fan_speed_help);
 
     ctk_help_heading(b, &i, "Type");
-    ctk_help_para(b, &i, __fan_control_type_help);
+    ctk_help_para(b, &i, "%s", __fan_control_type_help);
 
     ctk_help_heading(b, &i, "Cooling Target");
-    ctk_help_para(b, &i, __fan_cooling_target_help);
+    ctk_help_para(b, &i, "%s", __fan_cooling_target_help);
 
     ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-    ctk_help_para(b, &i, __enable_button_help);
+    ctk_help_para(b, &i, "%s", __enable_button_help);
 
     if ( ctk_thermal->show_fan_control_frame ) {
         ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-        ctk_help_para(b, &i, __apply_button_help);
+        ctk_help_para(b, &i, "%s", __apply_button_help);
 
         ctk_help_heading(b, &i, "Enable GPU Fan Settings");
-        ctk_help_para(b, &i, __reset_button_help);
+        ctk_help_para(b, &i, "%s", __reset_button_help);
     }
 done:
     ctk_help_finish(b);
