@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2006 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <stdlib.h> /* malloc */
@@ -122,6 +117,7 @@ GType ctk_vcs_get_type(void)
             sizeof(CtkVcs),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_vcs_type = g_type_register_static
@@ -236,7 +232,7 @@ static gboolean update_vcs_info(gpointer user_data)
  *
  **/
 
-GtkWidget * create_error_dialog(CtkVcs *ctk_object)
+static GtkWidget * create_error_dialog(CtkVcs *ctk_object)
 {
     GtkWidget *dialog;
     GtkWidget *image;
@@ -571,10 +567,9 @@ GtkWidget* ctk_vcs_new(NvCtrlAttributeHandle *handle,
     hbox_scroll = gtk_hbox_new(FALSE, 0);
     vbox_scroll = gtk_vbox_new(FALSE, 5);
     event = gtk_event_box_new();
+    ctk_force_text_colors_on_widget(event);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin),
                                    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-    gtk_widget_modify_fg(event, GTK_STATE_NORMAL, &(event->style->text[GTK_STATE_NORMAL]));
-    gtk_widget_modify_bg(event, GTK_STATE_NORMAL, &(event->style->base[GTK_STATE_NORMAL]));
     gtk_container_add(GTK_CONTAINER(event), hbox_scroll);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin),
                                           event);
@@ -628,7 +623,7 @@ GtkWidget* ctk_vcs_new(NvCtrlAttributeHandle *handle,
                                   &psu_str)  == NvCtrlSuccess)) {
         GtkWidget *vbox_padding;
 
-        /* Show the additonal queried information */
+        /* Show the additional queried information */
 
 
         /* Populate scrolling window with data */
