@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 /*
@@ -33,6 +28,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "ctkwindow.h"
 
@@ -48,9 +44,8 @@
 #include "ctkslimm.h"
 #include "ctkgpu.h"
 #include "ctkcolorcorrection.h"
+#include "ctkcolorcorrectionpage.h"
 #include "ctkxvideo.h"
-#include "ctkrandr.h"
-#include "ctkcursorshadow.h"
 #include "ctkopengl.h"
 #include "ctkglx.h"
 #include "ctkmultisample.h"
@@ -59,19 +54,26 @@
 #include "ctkclocks.h"
 #include "ctkvcs.h"
 #include "ctkpowersavings.h"
+#include "ctk3dvisionpro.h"
 
-#include "ctkdisplaydevice-crt.h"
-#include "ctkdisplaydevice-tv.h"
-#include "ctkdisplaydevice-dfp.h"
+#include "ctkdisplaydevice.h"
 
 #include "ctkdisplayconfig.h"
 #include "ctkserver.h"
 #include "ctkecc.h"
+#include "ctkvdpau.h"
+
+#include "ctkappprofile.h"
 
 #include "ctkhelp.h"
 #include "ctkevent.h"
 #include "ctkconstants.h"
 
+#include "msg.h"
+#include "common-utils.h"
+#include "query-assign.h"
+
+
 /* column enumeration */
 
 enum {
@@ -122,11 +124,15 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
                                 NvCtrlAttributeHandle *handle,
                                 CtkEvent *ctk_event,
                                 GtkTextTagTable *tag_table,
-                                UpdateDisplaysData *data);
+                                UpdateDisplaysData *data,
+                                ParsedAttribute *p);
 
 static void update_display_devices(GtkObject *object, gpointer arg1,
                                    gpointer user_data);
 
+static void ctk_window_destroyed(GtkObject *object, gpointer arg1,
+                                 gpointer user_data);
+
 
 static GObjectClass *parent_class;
 
@@ -152,6 +158,7 @@ GType ctk_window_get_type(void)
             sizeof(CtkWindow),
             0,                  /* n_preallocs */
             NULL,               /* instance_init */
+            NULL                /* value_table */
         };
 
         ctk_window_type = g_type_register_static
@@ -171,11 +178,9 @@ GType ctk_window_get_type(void)
 
 static void ctk_window_class_init(CtkWindowClass *ctk_window_class)
 {
-    GObjectClass *gobject_class;
     GtkObjectClass *object_class;
 
     object_class = (GtkObjectClass *) ctk_window_class;
-    gobject_class = (GObjectClass *) ctk_window_class;
 
     parent_class = g_type_class_peek_parent(ctk_window_class);
 
@@ -199,6 +204,19 @@ static void ctk_window_real_destroy(GtkObject *object)
 
 
 
+/* 
+ * ctk_window_destroyed() - called when ctk_window destroyed.
+ */
+
+static void ctk_window_destroyed(GtkObject *object, gpointer arg1,
+                                 gpointer user_data)
+{
+    CtkWindow *ctk_window = CTK_WINDOW(object);
+    save_settings_and_exit(ctk_window);
+} /* ctk_window_destroyed() */
+
+
+
 /*
  * close_button_clicked() - called when the "Quit" button is clicked.
  */
@@ -323,49 +341,39 @@ static void tree_selection_changed(GtkTreeSelection *selection,
 
 
 
-/*
- * tree_view_key_event() - callback for additional keyboard events we
- * want to track (space and Return) to expand and collapse collapsable
- * categories in the treeview.
+/* 
+ * row_activated_event() - callback for row-activated event
+ * - handles key presses automatically
+ * - allows the mouse to collapse/expand the menu even when the
+ *   expand button/triangle is not working.
  */
 
-static gboolean tree_view_key_event(GtkWidget *tree_view, GdkEvent *event, 
-                                    gpointer user_data)
+static void row_activated_event(GtkTreeView        *view,
+                                GtkTreePath        *path,
+                                GtkTreeViewColumn  *col,
+                                gpointer            user_data)
 {
-    GdkEventKey *key_event = (GdkEventKey *) event;
     CtkWindow *ctk_window = CTK_WINDOW(user_data);
-
-    if ((key_event->keyval == GDK_space) ||
-        (key_event->keyval == GDK_Return)) {
-        
-        GtkTreeSelection* selection;
-        GtkTreeModel *model;
-        GtkTreeIter iter;
-        GtkTreePath* path;
-
-        selection = gtk_tree_view_get_selection(ctk_window->treeview);
-        
-        if (!gtk_tree_selection_get_selected(selection, &model, &iter))
-            return FALSE;
-        
-        path = gtk_tree_model_get_path(model, &iter);
-
-        if (gtk_tree_view_row_expanded(ctk_window->treeview, path)) {
-            gtk_tree_view_collapse_row(ctk_window->treeview, path);
-        } else {
-            gtk_tree_view_expand_row(ctk_window->treeview, path, FALSE);
-        }
-
-        gtk_tree_path_free(path);
-
-        return TRUE;
+    
+    if (gtk_tree_view_row_expanded(ctk_window->treeview, path)) {
+        gtk_tree_view_collapse_row(ctk_window->treeview, path);
+    } else {
+        gtk_tree_view_expand_row(ctk_window->treeview, path, FALSE);
     }
-    
-    return FALSE;
-    
-} /* tree_view_key_event() */
+}
 
 
+
+static gboolean has_randr_gamma(NvCtrlAttributeHandle *handle)
+{
+    ReturnStatus ret;
+    int val;
+
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_ATTR_RANDR_GAMMA_AVAILABLE, &val);
+
+    return ((ret == NvCtrlSuccess) && (val == 1));
+}
+
 /*
  * ctk_window_new() - create a new CtkWindow widget
  */
@@ -516,10 +524,10 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     ctk_window->treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
     g_object_unref(ctk_window->tree_store);
 
-    /* catch keyboard events to the tree view */
+    /* Added row activated event to the tree view */
 
-    g_signal_connect(ctk_window->treeview, "key_press_event",
-                     G_CALLBACK(tree_view_key_event), GTK_OBJECT(ctk_window));
+    g_signal_connect(ctk_window->treeview, "row_activated",
+                     G_CALLBACK(row_activated_event), GTK_OBJECT(ctk_window));
 
     selection = gtk_tree_view_get_selection(ctk_window->treeview);
 
@@ -546,7 +554,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     /* create the container widget for the "pages" */
 
     hbox = gtk_hbox_new(FALSE, 0);
-    gtk_paned_pack2(GTK_PANED(hpane), hbox, TRUE, TRUE);
+    gtk_paned_pack2(GTK_PANED(hpane), hbox, TRUE, FALSE);
 
     ctk_window->page_viewer = hbox;
     ctk_window->page = NULL;
@@ -632,8 +640,10 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
-                           CTK_WINDOW_HELP_COLUMN, 
-                           ctk_screen_create_help(tag_table, screen_name), -1);
+                           CTK_WINDOW_HELP_COLUMN,
+                           ctk_screen_create_help(tag_table, CTK_SCREEN(child),
+                                                  screen_name),
+                           -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_CONFIG_FILE_ATTRIBUTES_FUNC_COLUMN,
                            NULL, -1);
@@ -650,15 +660,21 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
             }
         }
 
-        /* color correction */
+        /*
+         * color correction, if RandR per-CRTC color correction is not
+         * available
+         */
 
-        child = ctk_color_correction_new(screen_handle, ctk_config,
-                                         ctk_window->attribute_list,
-                                         ctk_event);
-        if (child) {
-            help = ctk_color_correction_create_help(tag_table);
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "X Server Color Correction", NULL, NULL, NULL);
+        if (!has_randr_gamma(screen_handle)) {
+
+            child = ctk_color_correction_page_new(screen_handle, ctk_config,
+                                                  ctk_window->attribute_list,
+                                                  ctk_event);
+            if (child) {
+                help = ctk_color_correction_page_create_help(tag_table);
+                add_page(child, help, ctk_window, &iter, NULL,
+                         "X Server Color Correction", NULL, NULL, NULL);
+            }
         }
 
         /* xvideo settings  */
@@ -670,25 +686,6 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                      "X Server XVideo Settings", NULL, NULL, NULL);
         }
 
-        /* randr settings */
-
-        child = ctk_randr_new(screen_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_randr_create_help(tag_table, CTK_RANDR(child));
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "Rotation Settings", NULL, NULL, NULL);
-        }
-
-        /* cursor shadow */
-
-        child = ctk_cursor_shadow_new(screen_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_cursor_shadow_create_help(tag_table,
-                                                 CTK_CURSOR_SHADOW(child));
-            add_page(child, help, ctk_window, &iter, NULL, "Cursor Shadow",
-                     NULL, NULL, NULL);
-        }
-
         /* opengl settings */
 
         child = ctk_opengl_new(screen_handle, ctk_config, ctk_event);
@@ -720,6 +717,14 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         }
 
 
+        /* VDPAU Information */
+        child = ctk_vdpau_new(screen_handle, ctk_config, ctk_event);
+        if (child) {
+            help = ctk_vdpau_create_help(tag_table, CTK_VDPAU(child));
+            add_page(child, help, ctk_window, &iter, NULL, "VDPAU Information",
+                     NULL, NULL, NULL);
+        }
+        
         /* gvo (Graphics To Video Out) */
         
         child = ctk_gvo_new(screen_handle, ctk_config, ctk_event);
@@ -798,10 +803,10 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                            NULL, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_SELECT_WIDGET_FUNC_COLUMN,
-                           ctk_gpu_start_timer, -1);
+                           ctk_gpu_page_select, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_UNSELECT_WIDGET_FUNC_COLUMN,
-                           ctk_gpu_stop_timer, -1);
+                           ctk_gpu_page_unselect, -1);
 
         /* power savings */
 
@@ -846,26 +851,31 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                      NULL, ctk_ecc_start_timer, ctk_ecc_stop_timer);
         }
         /* display devices */
-        data = (UpdateDisplaysData *)calloc(1, sizeof(UpdateDisplaysData));
+        data = calloc(1, sizeof(UpdateDisplaysData));
         data->window = ctk_window;
         data->event = ctk_event;
         data->gpu_handle = gpu_handle;
         data->parent_iter = iter;
         data->tag_table = tag_table;
-        
+
         g_signal_connect(G_OBJECT(ctk_event),
                          CTK_EVENT_NAME(NV_CTRL_PROBE_DISPLAYS),
                          G_CALLBACK(update_display_devices),
-                         (gpointer) data);            
-        
-        add_display_devices(ctk_window, &iter, gpu_handle, ctk_event,
-                            tag_table, data);
+                         (gpointer) data);
+
+        g_signal_connect(G_OBJECT(ctk_event),
+                         CTK_EVENT_NAME(NV_CTRL_MODE_SET_EVENT),
+                         G_CALLBACK(update_display_devices),
+                         (gpointer) data);
+
+        add_display_devices(ctk_window, &iter, gpu_handle, ctk_event, tag_table,
+                            data, ctk_window->attribute_list);
     }
 
     /* add the per-vcs (e.g. Quadro Plex) entries into the tree model */
 
     for (i = 0; i < h->targets[VCS_TARGET].n; i++) {
-        
+
         gchar *vcs_product_name;
         gchar *vcs_name;
         GtkWidget *child;
@@ -876,23 +886,24 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
         /* create the vcs entry name */
 
-        ret = NvCtrlGetStringDisplayAttribute(vcs_handle, 0,
-                                              NV_CTRL_STRING_VCSC_PRODUCT_NAME,
-                                              &vcs_product_name);
+        ret = NvCtrlGetStringAttribute(vcs_handle,
+                                       NV_CTRL_STRING_VCSC_PRODUCT_NAME,
+                                       &vcs_product_name);
         if (ret == NvCtrlSuccess && vcs_product_name) {
             vcs_name = g_strdup_printf("VCS %d - (%s)",
                                         NvCtrlGetTargetId(vcs_handle),
                                         vcs_product_name);
+            XFree(vcs_product_name);
         } else {
             vcs_name =  g_strdup_printf("VCS %d - (Unknown)",
                                         NvCtrlGetTargetId(vcs_handle));
         }
         if (!vcs_name) continue;
-        
+
         /* create the object for receiving NV-CONTROL events */
-        
+
         ctk_event = CTK_EVENT(ctk_event_new(vcs_handle));
-        
+
         /* create the vcs entry */
 
         gtk_tree_store_append(ctk_window->tree_store, &iter, NULL);
@@ -988,10 +999,39 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         break;
     }
 
+    /* add NVIDIA 3D VisionPro dongle configuration page */
+
+    for (i = 0; i < h->targets[NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET].n; i++) {
+        NvCtrlAttributeHandle *svp_handle = h->targets[NVIDIA_3D_VISION_PRO_TRANSCEIVER_TARGET].t[i].h;
+
+        if (!svp_handle) continue;
+
+        /* create the object for receiving NV-CONTROL events */
+
+        ctk_event = CTK_EVENT(ctk_event_new(svp_handle));
+
+        widget = ctk_3d_vision_pro_new(svp_handle, ctk_config,
+                                       ctk_window->attribute_list,
+                                       ctk_event);
+        if (!widget) continue;
+
+        help = ctk_3d_vision_pro_create_help(tag_table);
+        add_page(widget, help, ctk_window, NULL, NULL,
+                 "NVIDIA 3D VisionPro", ctk_3d_vision_pro_config_file_attributes,
+                 ctk_3d_vision_pro_select, ctk_3d_vision_pro_unselect);
+    }
+
+    /* app profile configuration */
+    widget = ctk_app_profile_new(ctk_config);
+
+    add_page(widget, ctk_app_profile_create_help(CTK_APP_PROFILE(widget), tag_table),
+             ctk_window, NULL, NULL, "Application Profiles",
+             NULL, NULL, NULL);
+
     /* nvidia-settings configuration */
 
     add_page(GTK_WIDGET(ctk_window->ctk_config),
-             ctk_config_create_help(tag_table),
+             ctk_config_create_help(ctk_config, tag_table),
              ctk_window, NULL, NULL, "nvidia-settings Configuration",
              NULL, NULL, NULL);
 
@@ -1052,12 +1092,96 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                                        GTK_POLICY_AUTOMATIC);
     }
 
+    /* Add callback when window destroyed */
+
+    g_signal_connect(G_OBJECT(ctk_window), "destroy",
+                     G_CALLBACK(ctk_window_destroyed), (gpointer) ctk_window);
+    
     return GTK_WIDGET(object);
 
 } /* ctk_window_new() */
 
 
 
+/*
+ * Set the currently active page to be the page that matches the
+ * specified label.
+ *
+ * Note that child pages of X screens and GPUs cannot be uniquely
+ * identified by their label alone (e.g., each GPU has a "PowerMizer"
+ * page).  To allow explicit control over which instance of a child
+ * page is selected, the label can have the optional format
+ *
+ *  "[PARENT LABEL], [CHILD LABEL]"
+ *
+ * E.g.,
+ *
+ *  "GPU 0 - (GeForce 7600 GT), PowerMizer"
+ */
+
+typedef struct {
+    CtkWindow *ctk_window;
+    const gchar *label;
+} ctk_window_set_active_page_args;
+
+static gboolean ctk_window_set_active_page_callback(GtkTreeModel *model,
+                                                    GtkTreePath *path,
+                                                    GtkTreeIter *iter,
+                                                    gpointer data)
+{
+    GtkTreeIter parent;
+    gboolean valid;
+    gchar *iter_label;
+    gchar *parent_label = "no parent";
+    gchar *qualified_label;
+    ctk_window_set_active_page_args *args = data;
+
+    /* get the parent's label, if there is a parent */
+
+    valid = gtk_tree_model_iter_parent(model, &parent, iter);
+    if (valid) {
+        gtk_tree_model_get(model, &parent,
+                           CTK_WINDOW_LABEL_COLUMN, &parent_label, -1);
+    }
+
+    gtk_tree_model_get(model, iter, CTK_WINDOW_LABEL_COLUMN, &iter_label, -1);
+
+    qualified_label = nvstrcat(parent_label, ", ", iter_label, NULL);
+
+    if ((strcmp(iter_label, args->label) == 0) ||
+        (strcmp(qualified_label, args->label) == 0)) {
+        GtkTreeSelection* selection;
+        selection = gtk_tree_view_get_selection(args->ctk_window->treeview);
+        gtk_tree_selection_select_iter(selection, iter);
+        free(qualified_label);
+        return TRUE; /* stop walking the tree */
+    }
+
+    free(qualified_label);
+
+    return FALSE; /* keep walking the tree */
+}
+
+
+void ctk_window_set_active_page(CtkWindow *ctk_window, const gchar *label)
+{
+    GtkTreeModel *model = GTK_TREE_MODEL(ctk_window->tree_store);
+
+    ctk_window_set_active_page_args args;
+
+    if (!label) return;
+
+    args.ctk_window = ctk_window;
+    args.label = label;
+
+    gtk_tree_model_foreach(model,
+                           ctk_window_set_active_page_callback,
+                           &args);
+
+} /* ctk_window_set_active_page() */
+
+
+
 /*
  * add_page() - add a new page to ctk_window's tree_store, using iter
  * as a parent; the new child iter is written in pchild_iter, if
@@ -1079,15 +1203,19 @@ static void add_page(GtkWidget *widget, GtkTextBuffer *help,
     if (!child_iter) child_iter = &tmp_child_iter;
 
     /*
-     * add another reference to the widget, so that it doesn't get
-     * destroyed the next time it gets hidden (removed from the page
-     * viewer).
+     * Add a reference to the object and sink (remove) the floating (gtk)
+     * reference.  This sink needs to happen before the page gets packed
+     * to ensure that we become the propper owner of these widgets.  This way,
+     * page will not be destroyed when they are hidden (removed from the page
+     * viewer), and we can properly destroy them when needed (for example, the
+     * display device pages are destroyed/recreated on hotplug events.)
      */
-    
-    gtk_object_ref(GTK_OBJECT(widget));
-    
+
+    g_object_ref(G_OBJECT(widget));
+    gtk_object_sink(GTK_OBJECT(widget));
+
     gtk_tree_store_append(ctk_window->tree_store, child_iter, iter);
-    
+
     gtk_tree_store_set(ctk_window->tree_store, child_iter,
                        CTK_WINDOW_LABEL_COLUMN, label, -1);
     gtk_tree_store_set(ctk_window->tree_store, child_iter,
@@ -1205,26 +1333,34 @@ static void quit_response(GtkWidget *button, gint response, gpointer user_data)
  * saved to the config file.
  */
 
+static gboolean add_special_config_file_attributes_callback(GtkTreeModel *model,
+                                                            GtkTreePath *path,
+                                                            GtkTreeIter *iter,
+                                                            gpointer data)
+{
+    GtkWidget *widget;
+    config_file_attributes_func_t func;
+    CtkWindow *ctk_window = data;
+
+    gtk_tree_model_get(model, iter,
+                       CTK_WINDOW_WIDGET_COLUMN, &widget,
+                       CTK_WINDOW_CONFIG_FILE_ATTRIBUTES_FUNC_COLUMN,
+                       &func, -1);
+
+    if (func) (*func)(widget, ctk_window->attribute_list);
+
+    return FALSE; /* keep iterating over nodes in the tree */
+}
+
+
 void add_special_config_file_attributes(CtkWindow *ctk_window)
 {
     GtkTreeModel *model = GTK_TREE_MODEL(ctk_window->tree_store);
-    GtkTreeIter   iter;
-    gboolean valid;
-    config_file_attributes_func_t func;
-    GtkWidget *widget;
-    
-    valid = gtk_tree_model_get_iter_first(model, &iter);
-    while (valid) {
-        
-        gtk_tree_model_get(model, &iter,
-                           CTK_WINDOW_WIDGET_COLUMN, &widget,
-                           CTK_WINDOW_CONFIG_FILE_ATTRIBUTES_FUNC_COLUMN,
-                           &func, -1);
-        
-        if (func) (*func)(widget, ctk_window->attribute_list);
-        
-        valid = gtk_tree_model_iter_next(model, &iter);
-    }
+
+    gtk_tree_model_foreach(model,
+                           add_special_config_file_attributes_callback,
+                           ctk_window);
+
 } /* add_special_config_file_attributes() */
 
 
@@ -1234,107 +1370,158 @@ void add_special_config_file_attributes(CtkWindow *ctk_window)
  */
 
 static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
-                                NvCtrlAttributeHandle *handle,
-                                CtkEvent *ctk_event,
+                                NvCtrlAttributeHandle *gpu_handle,
+                                CtkEvent *ctk_event_gpu,
                                 GtkTextTagTable *tag_table,
-                                UpdateDisplaysData *data)
+                                UpdateDisplaysData *data,
+                                ParsedAttribute *p)
 {
-    GtkWidget *widget;
     GtkTextBuffer *help;
     ReturnStatus ret;
-    int i, connected, n, mask;
-    char *name;
-    char *type;
-    gchar *title;
-    
-    
-    /* retrieve the connected display device mask */
+    int *pData;
+    int len;
+    int i;
+    NvCtrlAttributeHandle *display_handle;
 
-    ret = NvCtrlGetAttribute(handle, NV_CTRL_CONNECTED_DISPLAYS, &connected);
-    if (ret != NvCtrlSuccess) {
+
+    /* retrieve the list of connected display devices */
+
+    ret = NvCtrlGetBinaryAttribute(gpu_handle, 0,
+                                   NV_CTRL_BINARY_DATA_DISPLAYS_CONNECTED_TO_GPU,
+                                   (unsigned char **)(&pData), &len);
+    if ((ret != NvCtrlSuccess) || (pData[0] <= 0)) {
         return;
     }
 
-    /* count how many display devices are connected */
-
-    for (i = 0, n = 0; i < 24; i++) {
-        if (connected & (1 << i)) n++;
-    }
-    if (n == 0) {
-        return;
-    }
-    
-
-    if (data->display_iters) {
-        free(data->display_iters);
-    }
-    data->display_iters =
-        (GtkTreeIter *)calloc(1, n * sizeof(GtkTreeIter));
+    nvfree(data->display_iters);
+    data->display_iters = calloc(pData[0], sizeof(GtkTreeIter));
     data->num_displays = 0;
     if (!data->display_iters) return;
 
 
     /*
-     * create pages for each of the display devices driven by this handle.
+     * create pages for each of the display devices driven by this (gpu)
+     * handle.
      */
 
-    for (i = 0; i < 24; i++) {
-        mask = (1 << i);
-        if (!(mask & connected)) continue;
+    for (i = 0; i < pData[0]; i++) {
+        int display_id = pData[i+1];
+        char *logName;
+        char *typeBaseName;
+        char *typeIdName;
+        GtkWidget *widget;
+        gchar *title;
+        CtkEvent *ctk_event;
+        CtrlHandles *handles = ctk_window->ctk_config->pCtrlHandles;
 
-        type = display_device_mask_to_display_device_name(mask);
-        
-        ret =
-            NvCtrlGetStringDisplayAttribute(handle, mask,
-                                            NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
-                                            &name);
-        
-        if ((ret != NvCtrlSuccess) || (!name)) {
-            title = g_strdup_printf("%s - (Unknown)", type);
-        } else {
-            title = g_strdup_printf("%s - (%s)", type, name);
-            XFree(name);
+        /*
+         * Get the ctrl handle that was passed into ctk_main so that updated
+         * backend color slider values, cached in the handle itself, can be
+         * saved to the RC file when the UI is closed.
+         */
+
+        display_handle = nv_get_target_handle(handles,
+                                              NV_CTRL_TARGET_TYPE_DISPLAY,
+                                              display_id);
+
+        if (!display_handle) {
+            display_handle =
+                nv_add_target(handles, NvCtrlGetDisplayPtr(gpu_handle),
+                              NV_CTRL_TARGET_TYPE_DISPLAY, display_id);
+
+            if (!display_handle) {
+                continue;
+            }
         }
-        free(type);
-        
-        if (mask & CTK_DISPLAY_DEVICE_CRT_MASK) {
-            
-            widget = ctk_display_device_crt_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_crt_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_CRT(widget));
-            
-        } else if (mask & CTK_DISPLAY_DEVICE_TV_MASK) {
-            
-            widget = ctk_display_device_tv_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_tv_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_TV(widget));
-            
-        } else if (mask & CTK_DISPLAY_DEVICE_DFP_MASK) {
-            
-            widget = ctk_display_device_dfp_new
-                (handle, ctk_window->ctk_config, ctk_event, mask, title);
-            
-            help = ctk_display_device_dfp_create_help
-                (tag_table, CTK_DISPLAY_DEVICE_DFP(widget));
-            
-        } else {
-            g_free(title);
+
+        /*
+         * Rebuild Sub-systems of display handle
+         */
+        NvCtrlAttributeRebuildSubsystems(display_handle,
+                                         NV_CTRL_ATTRIBUTES_ALL_SUBSYSTEMS);
+
+        ctk_event = CTK_EVENT(ctk_event_new(display_handle));
+
+        /* Query display's names */
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_BASENAME,
+                                       &typeBaseName);
+        if (ret != NvCtrlSuccess) {
+            NvCtrlAttributeClose(display_handle);
             continue;
         }
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
+                                       &logName);
+        if (ret != NvCtrlSuccess) {
+            logName = NULL;
+        }
+        ret = NvCtrlGetStringAttribute(display_handle,
+                                       NV_CTRL_STRING_DISPLAY_NAME_TYPE_ID,
+                                       &typeIdName);
+        if (ret != NvCtrlSuccess) {
+            typeIdName = NULL;
+        }
 
-        add_page(widget, help, ctk_window, iter,
-                 &(data->display_iters[data->num_displays]), title,
-                 NULL, NULL, NULL);
+        if (!logName && !typeIdName) {
+            title = g_strdup_printf("DPY-%d - (Unknown)", display_id);
+        } else {
+            title = g_strdup_printf("%s - (%s)", typeIdName, logName);
+        }
+        XFree(logName);
+        XFree(typeIdName);
+
+        /* Create the page for the display */
+        widget = ctk_display_device_new(display_handle,
+                                        ctk_window->ctk_config, ctk_event,
+                                        ctk_event_gpu,
+                                        title, typeBaseName, p);
+        if (widget == NULL) {
+            NvCtrlAttributeClose(display_handle);
+        } else {
+            help = ctk_display_device_create_help(tag_table,
+                                                  CTK_DISPLAY_DEVICE(widget));
+            add_page(widget, help, ctk_window, iter,
+                     &(data->display_iters[data->num_displays]), title,
+                     NULL, NULL, NULL);
+            data->num_displays++;
+        }
+
+        XFree(typeBaseName);
         g_free(title);
-        data->num_displays++;
     }
 } /* add_display_devices() */
 
 
+/*
+ * Select display page whose name is passed through 'name' parameter
+ */
+static void select_display_page(UpdateDisplaysData *data, gchar *name)
+{
+    CtkWindow *ctk_window = data->window;
+    GtkTreeSelection *tree_selection =
+        gtk_tree_view_get_selection(ctk_window->treeview);
+    gint num_displays = data->num_displays;
+
+    while (num_displays) {
+        GtkTreeIter *iter = &(data->display_iters[num_displays -1]);
+        CtkDisplayDevice *ctk_obj = NULL;
+        GtkWidget *widget = NULL;
+
+        gtk_tree_model_get(GTK_TREE_MODEL(ctk_window->tree_store), iter,
+                           CTK_WINDOW_WIDGET_COLUMN, &widget, -1);
+
+        ctk_obj = CTK_DISPLAY_DEVICE(widget);
+
+        if (strcmp(name, ctk_obj->name) == 0) {
+            gtk_tree_selection_select_iter(tree_selection, iter);
+            break;
+        }
+
+        num_displays--;
+    }
+
+}
 
 /*
  * update_display_devices() - Callback handler for the NV_CTRL_PROBE_DISPLAYS
@@ -1349,7 +1536,6 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     UpdateDisplaysData *data = (UpdateDisplaysData *) user_data;
 
     CtkWindow *ctk_window = data->window;
-    CtkEvent *ctk_event = data->event;
     NvCtrlAttributeHandle *gpu_handle = data->gpu_handle;
     GtkTreeIter parent_iter = data->parent_iter;
     GtkTextTagTable *tag_table = data->tag_table;
@@ -1357,6 +1543,8 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     gboolean parent_expanded;
     GtkTreeSelection *tree_selection =
         gtk_tree_view_get_selection(ctk_window->treeview);
+    GtkWidget *widget;
+    gchar *selected_display_name = NULL;
 
 
     /* Keep track if the parent row is expanded */
@@ -1371,17 +1559,23 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
     while (data->num_displays) {
 
         GtkTreeIter *iter = &(data->display_iters[data->num_displays -1]);
-        GtkWidget *widget;
+        gboolean is_selected = FALSE;
 
         /* Select the parent (GPU) iter if we're removing the selected page */
         if (gtk_tree_selection_iter_is_selected(tree_selection, iter)) {
+            is_selected = TRUE;
             gtk_tree_selection_select_iter(tree_selection, &parent_iter);
         }
 
         /* Remove the entry */
         gtk_tree_model_get(GTK_TREE_MODEL(ctk_window->tree_store), iter,
                            CTK_WINDOW_WIDGET_COLUMN, &widget, -1);
-       
+
+        if (is_selected) {
+            CtkDisplayDevice *ctk_obj = CTK_DISPLAY_DEVICE(widget);
+            selected_display_name = g_strdup(ctk_obj->name);
+        }
+
         gtk_tree_store_remove(ctk_window->tree_store, iter);
 
         /* unref the page so we don't leak memory */
@@ -1389,14 +1583,24 @@ static void update_display_devices(GtkObject *object, gpointer arg1,
 
         data->num_displays--;
     }
-    
+
     /* Add back all the connected display devices */
-    add_display_devices(ctk_window, &parent_iter, gpu_handle, ctk_event,
-                        tag_table, data);
+
+    gtk_tree_model_get(GTK_TREE_MODEL(ctk_window->tree_store), &parent_iter,
+                       CTK_WINDOW_WIDGET_COLUMN, &widget, -1);
+
+    add_display_devices(ctk_window, &parent_iter, gpu_handle,
+                        CTK_GPU(widget)->ctk_event,
+                        tag_table, data, ctk_window->attribute_list);
 
     /* Expand the GPU entry if it used to be */
     if (parent_expanded) {
         gtk_tree_view_expand_row(ctk_window->treeview, parent_path, TRUE);
     }
 
+    if (selected_display_name) {
+        select_display_page(data, selected_display_name);
+        g_free(selected_display_name);
+    }
+
 } /* update_display_devices() */
