@@ -4,26 +4,24 @@
  *
  * Copyright (C) 2004 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include "msg.h"
 #include "command-line.h"
+#include "common-utils.h"
+
+#include <NvCtrlAttributes.h>
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -70,6 +68,26 @@ void nv_error_msg(const char *fmt, ...)
 
 
 
+/*
+ * nv_deprecated_msg() - print a deprecation message, nicely formatted using
+ * the format() function.
+ *
+ * This function should be used for all deprecation messages.
+ */
+
+void nv_deprecated_msg(const char *fmt, ...)
+{
+    if (__verbosity < VERBOSITY_DEPRECATED) return;
+
+    fprintf(stderr, "\n");
+
+    NV_FORMAT(stderr, "DEPRECATED: ", fmt, False);
+
+    fprintf(stderr, "\n");
+}
+
+
+
 /*
  * nv_warning_msg() - print a warning message, nicely formatted using
  * the format() function.
@@ -168,7 +186,7 @@ static void format(FILE *stream, const char *prefix,
 
     if (prefix) {
         prefix_len = strlen(prefix);
-        local_prefix = malloc(prefix_len+1);
+        local_prefix = nvalloc(prefix_len+1);
         strcpy(local_prefix, prefix);
     } else {
         prefix_len = 0;
@@ -218,7 +236,7 @@ static void format(FILE *stream, const char *prefix,
          */
 
         len = b-a;
-        line = malloc(len+1);
+        line = nvalloc(len+1);
         strncpy(line, a, len);
         line[len] = '\0';
         if (local_prefix) {
